package italo.pacman;

import italo.pacman.constants.GUIConstants;
import italo.pacman.controller.HandleMenu;
import italo.pacman.controller.HandleSistema;
import italo.pacman.gui.ContainerModel;
import italo.pacman.gui.ContainerPanel;
import italo.pacman.gui.GUIFacade;
import italo.pacman.gui.PersonalPanel;
import italo.pacman.gui.input.Input;
import italo.pacman.gui.input.InputKey;
import italo.pacman.gui.menu.event.MenuActionListener;
import italo.pacman.logic.Sistema;
import italo.pacman.logic.SistemaFacade;
import italo.pacman.logic.event.SistemaListener;
import italo.pacman.multimidia.MultimidiaFacade;

import java.awt.Container;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;

import javax.swing.JApplet;

public class MainApplet extends JApplet implements Runnable, ContainerModel, GUIConstants {

	private static final long serialVersionUID = 1L;
	
	private ContainerPanel containerPanel;
	private Input input;
	private MenuActionListener menuListener;	
	private SistemaListener sistemaListener;
	
	public MainApplet() {
		super.setContentPane( getContainerPanel() );
		MultimidiaFacade.getInstance().initialize( this );		
		GUIFacade.getInstance().setContainerModel( this );
	}
	
	public void init() {		
		super.setSize(FRAME_WIDTH, FRAME_HEIGHT);	
		super.setFocusable(true);	
		super.setVisible(true);						
		new Thread(this).start();
	}
	
	public void run() {
		Sistema sistema = SistemaFacade.getInstance().getCurrentSistema();
		sistema.setSistemaListener( getSistemaListener() );		
		sistema.abertura();
		sistema.load();				
		sistema.play();
		sistema.removeSistemaListener();
	}
	
	public void start() {
		if(SistemaFacade.getInstance().getCurrentGame().isPause())
			SistemaFacade.getInstance().getCurrentGame().gamePause();		
	}
	
	public void stop() {
		if(!SistemaFacade.getInstance().getCurrentGame().isPause()) 
			SistemaFacade.getInstance().getCurrentGame().gamePause();		
	}
		
	public void destroy() {
		getMenuActionListener().exit();
	}
	
	public PersonalPanel getPersonalPanel() {
		return containerPanel.getPainelPersonalizado();
	}
	
	public Container getContainer2() {
		return this;
	}
	
	public void reinit() {
		getMenuActionListener().reinit();
	}
	
	public URL resourceURL(String file) {
		try {
			return new URL(getCodeBase(), file);
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	public String resourcePath(String file) {
		try {
			return this.resourceURL( file ).toURI().getPath();
		} catch (URISyntaxException e) {
			e.printStackTrace();
		}
		return null;
	}
		
	public boolean end() {		
		return SistemaFacade.getInstance().getCurrentSistema().finalized();
	}
		
	public Input getInput() {
		if(input == null)
			input = new InputKey( this );		
		return input;
	}
			
	public ContainerPanel getContainerPanel() {
		if(containerPanel == null) {
			containerPanel = new ContainerPanel(this);	
			containerPanel.setMenuActionListener( getMenuActionListener() ); 
		}
		return containerPanel;
	}
	
	public MenuActionListener getMenuActionListener() {
		if(menuListener == null) {
			menuListener = new HandleMenu();
		}
		return menuListener;
	}
	
	public SistemaListener getSistemaListener() {
		if(sistemaListener == null) {
			sistemaListener = new HandleSistema();
		}
		return sistemaListener;
	}
			
}
