package italo.pacman;

import italo.pacman.constants.GUIConstants;
import italo.pacman.controller.HandleMenu;
import italo.pacman.controller.HandleSistema;
import italo.pacman.gui.ContainerModel;
import italo.pacman.gui.ContainerPanel;
import italo.pacman.gui.GUIFacade;
import italo.pacman.gui.PersonalPanel;
import italo.pacman.gui.input.Input;
import italo.pacman.gui.input.InputKey;
import italo.pacman.gui.menu.event.MenuActionListener;
import italo.pacman.logic.Sistema;
import italo.pacman.logic.SistemaFacade;
import italo.pacman.logic.event.SistemaListener;
import italo.pacman.multimidia.MultimidiaFacade;

import java.applet.AudioClip;
import java.awt.Container;
import java.awt.Image;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JApplet;
import javax.swing.JFrame;

public class MainDesktop extends JFrame implements ContainerModel, GUIConstants {

	private static final long serialVersionUID = 1L;

	private ContainerPanel containerPanel;
	private Input input;
	private MenuActionListener menuListener;
	private SistemaListener sistemaListener;
		
	public MainDesktop() {
		super.setContentPane( getContainerPanel() );
		super.setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );
		super.setSize( FRAME_WIDTH, FRAME_HEIGHT );
		super.setResizable( false );
		super.setLocationRelativeTo( this );			
	}
	
	public static void main(String[] args) {
		MainDesktop desktop = new MainDesktop();
		desktop.setVisible(true);
		MultimidiaFacade.getInstance().initialize( desktop );
		GUIFacade.getInstance().setContainerModel( desktop );	
		Sistema sistema = SistemaFacade.getInstance().getCurrentSistema();		
		sistema.setSistemaListener( desktop.getSistemaListener() );		
		sistema.abertura();				
		sistema.load();	
		sistema.play();
		sistema.removeSistemaListener();		
	}
		
	public PersonalPanel getPersonalPanel() {
		return containerPanel.getPainelPersonalizado();
	}
	
	public Container getContainer2() {
		return this;
	}
	
	public URL resourceURL(String file) {
		try {
			return new File( file ).toURI().toURL();
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	public String resourcePath(String file) {		
		return file;
	}
	
	public Image getImage(URL url) {
		return new ImageIcon( url ).getImage();
	}
	
	public AudioClip getAudioClip(URL url) {
		return JApplet.newAudioClip( url );
	}
	
	public boolean end() {		
		return SistemaFacade.getInstance().getCurrentSistema().finalized();
	}
	
	public Input getInput() {
		if(input == null)
			input = new InputKey( this );		
		return input;
	}
			
	public ContainerPanel getContainerPanel() {
		if(containerPanel == null) {
			containerPanel = new ContainerPanel(this);	
			containerPanel.setMenuActionListener( getMenuActionListener() ); 
		}
		return containerPanel;
	}
	
	public MenuActionListener getMenuActionListener() {
		if(menuListener == null) {
			menuListener = new HandleMenu();
		}
		return menuListener;
	}
	
	public SistemaListener getSistemaListener() {
		if(sistemaListener == null) {
			sistemaListener = new HandleSistema();
		}
		return sistemaListener;
	}

}
