package italo.pacman.adapter.figure;

import italo.pacman.utilities.Utilities;

abstract class AbstractFigure implements Figure {

	protected int x;
	protected int y;
	protected int width;
	protected int height;
		
	protected AbstractFigure(int x, int y, int width, int height) {		
		this.x = x;
		this.y = y;
		this.width = width;
		this.height = height;
	}
		
	public double distancia(Figure f) {
		int xCentro = getXCenter();
		int yCentro = getYCenter();
		int xFCentro = f.getXCenter();
		int yFCentro = f.getYCenter();
		return Utilities.distance(xCentro, yCentro, xFCentro, yFCentro);
	}
	
	public synchronized void redimensionar(int newWidth, int newHeight) {
		if(width == 0 || height == 0) {
			width = newWidth;
			height = newHeight;
		} else {
			double fatorL = ((double)newWidth)/((double)width);
			double fatorA = ((double)newHeight)/((double)height);		
			x = (int) (((double)x)*fatorL);
			y = (int) (((double)y)*fatorA);
			width = (int) (((double)width)*fatorL);
			height = (int) (((double)height)*fatorA);
		} 
	}
			
	public boolean colidiu(Figure f) {
		int x1 = f.getX();
		int y1 = f.getY();
		int x2 = f.getX()+f.getWidth();
		int y2 = f.getY()+f.getHeight();
		int[][] pontos = { {x1, y1}, {x1, y2},	{x2, y1}, {x2, y2} };
		for(int i = 0; i < pontos.length; i++)
			if(pontoContido(pontos[i][0], pontos[i][1]))
				return true;
		return false;
	}
	
	public synchronized boolean pontoContido(int xPonto, int yPonto) {
		int x1 = x;
		int y1 = y;
		int x2 = x + width;
		int y2 = y + height;
		return (x >= x1 && x <= x2 && y >= y1 && y <= y2);
			
	}
			
	public synchronized int getXCenter() {
		return (x+(width/2));
	}
	
	public synchronized int getYCenter() {
		return (y+(height/2));
	}
	
	public synchronized int getX() {
		return x;
	}

	public synchronized void setX(int x) {
		this.x = x;
	}

	public synchronized int getY() {
		return y;
	}

	public synchronized void setY(int y) {
		this.y = y;
	}

	public synchronized int getWidth() {
		return width;
	}

	public synchronized void setWidth(int width) {
		this.width = width;
	}

	public synchronized int getHeight() {
		return height;
	}

	public synchronized void setHeight(int height) {
		this.height = height;
	}
	
	public String toString() {
		return "("+x+","+y+") -- LARGURA: "+width+";  ALTURA: "+height+"; ";
	}

}
