package italo.pacman.algorithm;

import java.util.HashSet;
import java.util.Set;

public class AlgorithmController {

	private static AlgorithmController singleton = null;
	
	private AlgorithmController() {}
	
	private Set<Algorithm> threadAlgorithms = new HashSet<Algorithm>();
	private Set<Algorithm> normalAlgorithms = new HashSet<Algorithm>();
	
	public static AlgorithmController getInstance() {
		if(singleton == null) {
			synchronized(AlgorithmController.class) {
				singleton = new AlgorithmController();
			}
		}
		return singleton;
	}
	
	public boolean addThreadAlgorithm(Algorithm algorithm) {
		return threadAlgorithms.add(algorithm);
	}
	
	public boolean removeThreadAlgorithm(Algorithm algorithm) {
		return threadAlgorithms.remove(algorithm);
	}
	
	public boolean addNormalAlgorithm(Algorithm algorithm) {
		return normalAlgorithms.add(algorithm);
	}
	
	public boolean removeNormalAlgorithm(Algorithm algorithm) {
		return normalAlgorithms.remove(algorithm);
	}

	public boolean execute(Algorithm algorithm) {
		boolean started = this.start(algorithm);
		if(!started) {
			boolean runed = this.run(algorithm);
			return runed;
		}
		return true;
	}
	
	public boolean run(Algorithm algorithm) {
		if(normalAlgorithms.contains(algorithm)) {
			algorithm.run();
			return true;
		}
		return false;	
	}
	
	public boolean start(Algorithm algorithm) {
		if(threadAlgorithms.contains(algorithm)) {
			algorithm.run();
			return true;
		}
		return false;
	}

	
}
