package italo.pacman.algorithm;

import italo.pacman.algorithm.pause.PauseController;


class Thread2 extends Thread {
		
	public final static int STATUS_READY = 2;
	public final static int STATUS_EXECUTING = 3;
	public final static int STATUS_WAITED = 4;
	public final static int STATUS_SLEEPING = 5;
	
	private Algorithm algoritmo;
	private int status;	

	public Thread2(String name, Algorithm algoritmo) {
		super(name);		
		this.algoritmo = algoritmo;
		this.status = STATUS_READY;	
	}
	
	public static String getStatusString(int status) {
		switch(status) {
			case STATUS_READY: return "STATUS PRONTO";
			case STATUS_EXECUTING: return "STATUS EXECUTANDO";
			case STATUS_WAITED: return "STATUS BLOQUEADO";
			case STATUS_SLEEPING: return "STATUS DORMINDO";
			default: return "STATUS INVALIDO";
		}
	}
	
	public void start() {		
		super.start();
		setStatus(STATUS_EXECUTING);
	}
	
	public void run() {		
		algoritmo.begined();
		addAlgorithmPauseController();
		do {
			if(algoritmo.pausable())
				while(algoritmo.pauseCondition())
					algoritmo.wait2();			
			algoritmo.execute();
		} while(algoritmo.executionCondition());
		removeAlgorithmPauseController();
		algoritmo.ended();		
		setStatus(STATUS_READY);
	}
	
	public void addAlgorithmPauseController() {
		if(algoritmo.pausable())
			PauseController.getInstance().addAlgoritmo( algoritmo );
	}
	
	public void removeAlgorithmPauseController() {
		if(algoritmo.pausable())
			PauseController.getInstance().removeAlgoritmo( algoritmo );
	}

	public int getStatus() {
		return status;
	}

	public void setStatus(int status) {
		this.status = status;
	}
						
}
