package italo.pacman.algorithm.pause;

import italo.pacman.algorithm.Algorithm;
import italo.pacman.algorithm.pause.event.PauseListener;

import java.util.HashSet;
import java.util.Set;

public class PauseController {

	private static PauseController singleton = null;
					
	private Set<PauseListener> listeners = new HashSet<PauseListener>();
	private Set<Algorithm> algoritmos = new HashSet<Algorithm>();	
	
	private PauseController() {}
	
	public static PauseController getInstance() {
		if(singleton == null) {
			synchronized(PauseController.class) {
				singleton = new PauseController();
			}
		}
		return singleton;
	}
	
	public boolean addPauseListener(PauseListener listener) {
		return listeners.add(listener);
	}
	
	public boolean removePauseListener(PauseListener listener) {
		return listeners.remove(listener);
	}
	
	public void continuarGame() {		
		notifyAlgoritmos();
		for(PauseListener listener:listeners)
			listener.continueGame();		
	}
	
	public void pausarGame() {
		for(PauseListener listener:listeners)
			listener.pauseGame();		
	}
		
	public boolean addAlgoritmo(Algorithm algoritmo) {
		return algoritmos.add(algoritmo);
	}
	
	public boolean removeAlgoritmo(Algorithm algoritmo) {
		return algoritmos.remove(algoritmo);
	}

	private void notifyAlgoritmos() {
		for(Algorithm algoritmo:algoritmos) 
			algoritmo.notify2();		
	}
	
}
