package italo.pacman.controller;

import italo.pacman.algorithm.pause.PauseController;
import italo.pacman.algorithm.pause.event.PauseListener;
import italo.pacman.constants.AudioConstants;
import italo.pacman.constants.ImageConstants;
import italo.pacman.constants.MessageConstants;
import italo.pacman.controller.algorithm.GameAlgorithm;
import italo.pacman.gui.GUIFacade;
import italo.pacman.load.LoadFacade;
import italo.pacman.logic.game.Fase;
import italo.pacman.logic.game.Game;
import italo.pacman.logic.game.attributes.GameAttributes;
import italo.pacman.logic.game.event.FaseListener;
import italo.pacman.logic.game.event.GameListener;
import italo.pacman.logic.game.event.ScoreListener;
import italo.pacman.logic.game.event.TrayListener;
import italo.pacman.multimidia.MultimidiaFacade;
import italo.pacman.multimidia.message.MessageController;
import italo.pacman.trick.Trick;
import italo.pacman.trick.TrickController;
import italo.pacman.trick.TrickListener;

import java.awt.Image;

public class HandleGame implements GameListener, MessageConstants {
	
	public final static boolean HIDE_MESSAGE = true;
	public final static boolean JOIN_MESSAGE = false;
	
	public final static int SHOW_PAUSE_MESSAGE_TIME = 0;
	public final static boolean HIDE_PAUSE_MESSAGE = false;
		
	private FaseListener handleFase = new HandleFase();	
	private ScoreListener handleScore = new HandleScore();
	private TrayListener handleTray = new HandleTray();
	
	private PauseListener pauseListener = new HandlePauseGame();
	
	private TrickListener crossWallTrick = new HandleCrossWallTrick();
	private TrickListener changeFaseTrick = new HandleChangeFaseTrick();
			
	public void initialize(Game game) {				
		GameAttributes gameAttributes = LoadFacade.loadGameAttributes();
		game.initialize(gameAttributes);			
		for(Fase fase:game.getFases()) {
			fase.getTray().setTrayListener(handleTray);
			fase.getTray().resize(); 
			fase.getTray().removeTrayListener();
		}		
		game.getScore().setScoreListener(handleScore);
		game.getScore().resize();
		game.getScore().removeScoreListener();
	}
	
	public void init(Game game) {
		Trick trick = TrickController.getInstance().getGameTrick(TrickController.CHAVE_TRUQUE_ATRAVESSAR_PAREDES);
		Trick trick2 = TrickController.getInstance().getGameTrick(TrickController.CHAVE_TRUQUE_MUDAR_DE_FASE);		
		trick.addTrickListener(crossWallTrick);
		trick2.addTrickListener(changeFaseTrick); 		
				
		game.init( GUIFacade.getInstance().getGamePicture() );	
		GUIFacade.getInstance().setCurrentPicture( 
				GUIFacade.getInstance().getGamePicture() );
	}
	
	public void play(Game game) {
		PauseController.getInstance().addPauseListener(pauseListener);
		GameAlgorithm algorithm = new GameAlgorithm(game);
		algorithm.run();
		PauseController.getInstance().removePauseListener(pauseListener);
	}
	
	public void finish(Game game) {
		Trick trick = TrickController.getInstance().getGameTrick(TrickController.CHAVE_TRUQUE_ATRAVESSAR_PAREDES);
		Trick trick2 = TrickController.getInstance().getGameTrick(TrickController.CHAVE_TRUQUE_MUDAR_DE_FASE);				
		trick.removeTrickListener(crossWallTrick);
		trick2.removeTrickListener(changeFaseTrick);	
	}

	public void begin(Game game) {		
		for(Fase fase:game.getFases()) 
			fase.setFaseListener(handleFase);		
		game.firstFase();
		game.getCurrentFase().init();		
	}
	
	public void end(Game game) {		
		for(Fase fase:game.getFases())
			fase.removeFaseListener();		
	}
		
	public void passouDeFase(Game game) {
		game.getCurrentFase().finish();
		if(game.isLastFase()) { 
			zerou(game);		
		} else {
			Image image = MultimidiaFacade.getInstance().getImagemMensagem(ImageConstants.MENSAGEM_PASSOU);
			MessageController.getInstance().desenharMensagem(
					image, game, PASSOU_MESSAGE_WIDTH_FACTOR, 
					PASSOU_MESSAGE_HEIGHT_FACTOR, PASSOU_MESSAGE_TIME,
					HIDE_MESSAGE, JOIN_MESSAGE);
			MultimidiaFacade.getInstance().executeAudio(AudioConstants.PASSOU);
			TimeController.getInstance().sleep(PASSOU_MESSAGE_TIME);
		}
	}

	public void perdeu(Game game) {
		game.getCurrentFase().finish();
		int novoNVidas = game.getLifeQuantity()-1;
		if(novoNVidas > 0) {				
			game.setLifeQuantity(novoNVidas);			
			String nVidas = String.valueOf(novoNVidas);
			Image image = MultimidiaFacade.getInstance().getImagemMensagemVida(nVidas);
			MessageController.getInstance().desenharMensagem(
					image, game, PERDEU_MESSAGE_WIDTH_FACTOR, 
					PERDEU_MESSAGE_HEIGHT_FACTOR, PERDEU_MESSAGE_TIME,
					HIDE_MESSAGE, JOIN_MESSAGE);
			MultimidiaFacade.getInstance().executeAudio(AudioConstants.OOH);
			TimeController.getInstance().sleep(OHH_MESSAGE_TIME);
			if(novoNVidas <= AudioConstants.AUDIOS_VIDA_QUANTITY) {
				MultimidiaFacade.getInstance().executeAudioVida( String.valueOf(novoNVidas) );
				TimeController.getInstance().sleep(PERDEU_MESSAGE_TIME);
			}
		} else {
			gameover(game);
		}		
	}

	public void zerou(Game game) {	
		Image image = MultimidiaFacade.getInstance().getImagemMensagem(ImageConstants.MENSAGEM_ZEROU);
		MessageController.getInstance().desenharMensagem(
				image, game, ZEROU_MESSAGE_WIDTH_FACTOR, 
				ZEROU_MESSAGE_HEIGHT_FACTOR, ZEROU_MESSAGE_TIME,
				HIDE_MESSAGE, JOIN_MESSAGE);
		MultimidiaFacade.getInstance().executeAudio(AudioConstants.WOODY);
		TimeController.getInstance().sleep(WOODY_MESSAGE_TIME);
		MultimidiaFacade.getInstance().executeAudio(AudioConstants.ZEROU);
		TimeController.getInstance().sleep(ZEROU_MESSAGE_TIME);
		game.finish();		
	}
	
	public void gameover(Game game) {
		Image image = MultimidiaFacade.getInstance().getImagemMensagem(ImageConstants.MENSAGEM_GAMEOVER);		
		MessageController.getInstance().desenharMensagem(
				image, game, GAMEOVER_MESSAGE_WIDTH_FACTOR, 
				GAMEOVER_MESSAGE_HEIGHT_FACTOR, GAMEOVER_MESSAGE_TIME,
				HIDE_MESSAGE, JOIN_MESSAGE);
		MultimidiaFacade.getInstance().executeAudio(AudioConstants.RISADAMALIGNA);
		TimeController.getInstance().sleep(RISADA_MALIGNA_MESSAGE_TIME);
		MultimidiaFacade.getInstance().executeAudio(AudioConstants.GAMEOVER);
		TimeController.getInstance().sleep(GAMEOVER_MESSAGE_TIME);
		game.finish();
	}
	
	public void pauseGame(Game game) {
		Image image = MultimidiaFacade.getInstance().getImagemMensagem(ImageConstants.MENSAGEM_PAUSE);
		MessageController.getInstance().desenharMensagem(image, game, 
				PAUSE_MESSAGE_WIDTH_FACTOR, PAUSE_MESSAGE_HEIGHT_FACTOR, 
				SHOW_PAUSE_MESSAGE_TIME, HIDE_PAUSE_MESSAGE, JOIN_MESSAGE);
	}
	
	public void continueGame(Game game) {
		MessageController.getInstance().apagarMensagem();
		PauseController.getInstance().continuarGame();
	}
	
}
