package italo.pacman.controller;

import italo.pacman.constants.AudioConstants;
import italo.pacman.constants.ImageConstants;
import italo.pacman.controller.algorithm.SistemaAlgorithm;
import italo.pacman.gui.GUIFacade;
import italo.pacman.gui.input.event.InputContainerListener;
import italo.pacman.gui.input.event.InputGameListener;
import italo.pacman.logic.Sistema;
import italo.pacman.logic.abertura.Abertura;
import italo.pacman.logic.abertura.event.AberturaListener;
import italo.pacman.logic.encerramento.Encerramento;
import italo.pacman.logic.encerramento.event.EncerramentoListener;
import italo.pacman.logic.event.SistemaListener;
import italo.pacman.logic.game.event.GameListener;
import italo.pacman.multimidia.MidiaLoad;
import italo.pacman.multimidia.MultimidiaFacade;

public class HandleSistema implements SistemaListener {

	private InputContainerListener handleEntradaJanela = new HandleContainerInput();
	private InputGameListener handleEntradaJogo = new HandleGameInput();
	private GameListener handleGame = new HandleGame();
	private AberturaListener handleAbertura = new HandleAbertura();
	private EncerramentoListener handleEncerramento = new HandleEncerramento();
			
	public void abertura(Sistema sistema) {
		Abertura abertura = sistema.getAbertura();
		abertura.setAberturaListener(handleAbertura);
		abertura.initialize();
		abertura.init();
		abertura.removeAberturaListener();
	}
	
	public void encerramento(Sistema sistema) {
		Encerramento encerramento = sistema.getEncerramento();
		encerramento.setEncerramentoListener(handleEncerramento);
		encerramento.initialize();
		encerramento.init();
		encerramento.removeEncerramentoListener();
	}
	
	public void load(Sistema sistema, MidiaLoad midiaLoad) {		
		MultimidiaFacade.getInstance().audiosLoad(AudioConstants.AUDIO_PATHS, midiaLoad);
		MultimidiaFacade.getInstance().imagesLoad(ImageConstants.IMAGE_PATHS, midiaLoad);
	}
	
	public void load(Sistema sistema) {		
		sistema.load(AudioConstants.AUDIO_PATHS.length + ImageConstants.IMAGE_PATHS.length);		
	}
	
	public void loading(Sistema sistema, int percentage) {
		GUIFacade.getInstance().repaint();
	}

	public void play(Sistema sistema) {
                GUIFacade.getInstance().repaint();
                
		SistemaAlgorithm algorithm = new SistemaAlgorithm("THREAD (SISTEMA)", sistema);                                
		algorithm.run();                
	}	
	
	public void begin(Sistema sistema) {
		GUIFacade.getInstance().getInput().setInputContainerListener(handleEntradaJanela);
		GUIFacade.getInstance().getInput().setInputGameListener(handleEntradaJogo);		
		sistema.getGame().setGameListener(handleGame);
		sistema.getGame().initialize();		
	}
	
	public void end(Sistema sistema) {
		GUIFacade.getInstance().getInput().removeInputContainerListener();						
		GUIFacade.getInstance().getInput().removeInputGameListener(); 
		sistema.getGame().removeGameListener();
	}
	
	public void finish(Sistema sistema) {
		if(sistema.getGame().isPause())
			sistema.getGame().gamePause();			
	}
		
}
