package italo.pacman.controller;

import italo.pacman.algorithm.time.TimeAlgorithm;
import italo.pacman.algorithm.time.TimeConditions;
import italo.pacman.logic.SistemaFacade;
import italo.pacman.logic.game.Game;

public class TimeController {
	
	public static TimeController singleton = null;
	
	private TimeController() {}
	
	public static TimeController getInstance() {
		if(singleton == null) {
			synchronized(TimeController.class) {
				singleton = new TimeController();
			}
		}
		return singleton;
	}
	
	public void sleep(int time, TimeConditions conditions) {		
		TimeAlgorithm algorithm = new TimeAlgorithm("THREAD (TIME CONTROLLER)", conditions, time);
		algorithm.start();
		algorithm.join();
	}
	
	public void sleep(int time) {
		Game game = SistemaFacade.getInstance().getCurrentGame();
		ControllerCountConditions conditions = 
			new ControllerCountConditions(game);
		this.sleep(time, conditions);
	}
			
	public class ControllerCountConditions implements TimeConditions {
		
		private Game game;
		
		public ControllerCountConditions(Game game) {			
			this.game = game;
		}
				
		public void timeBegined() {
			
		}
		
		public void timeEnded() {
			
		}
				
		public boolean executionCondition() {
			return !game.finalized();
		}

		public boolean pauseCondition() {
			return false;
		}
		
		public boolean pausable() {
			return false;
		}
		
	}


}
