package italo.pacman.controller.algorithm;

import italo.pacman.algorithm.AbstractAlgorithm;
import italo.pacman.constants.StateConstants;
import italo.pacman.logic.game.Fase;
import italo.pacman.logic.game.Game;
import italo.pacman.logic.game.Monster;
import italo.pacman.logic.game.Pacman;
import italo.pacman.logic.game.Tray;
import italo.pacman.multimidia.Animation;
import italo.pacman.multimidia.MultimidiaFacade;
import italo.pacman.multimidia.midia.Midia;
import italo.pacman.multimidia.midia.image.ImagemComeCome;
import italo.pacman.multimidia.midia.image.ImagemDirecao;
import italo.pacman.multimidia.midia.image.ImagemEstado;
import italo.pacman.multimidia.midia.image.ImagemID;
import italo.pacman.multimidia.midia.image.ImagemMonstrinho;
import italo.pacman.multimidia.midia.image.ImagemPersonagem;

import java.util.List;

public class FaseAlgorithm extends AbstractAlgorithm {

	private Game game;
	private Fase fase;
	
	private PacmanAlgorithm algoritmoPacman;
	private MonsterAlgorithm[] algoritmosMonster;
	private PictureAlgorithm algoritmoPicture;
		
	public FaseAlgorithm(Game game, Fase fase) {
		super("THREAD (FASE)");		
		this.game = game;
		this.fase = fase;
	}
				
	public boolean pauseCondition() {
		return false;
	}
		
	public boolean pausable() {
		return false;
	}
	
	public void begined() {
		fase.begin();
	}
	
	public void ended() {		
		fase.end();
	}
	
	public boolean executionCondition() {
		return false;
	}
	
	public void execute() {
		algoritmoPicture.start();
		algoritmoPacman.start();
		for(MonsterAlgorithm algoritmoMonster:algoritmosMonster)
			algoritmoMonster.start();		
		algoritmoPicture.join();
		algoritmoPacman.join();
		for(MonsterAlgorithm algoritmoMonster:algoritmosMonster)
			algoritmoMonster.join();	
	}

	
	public void init() {				
		Tray tray = fase.getTray();
		Pacman pacman = tray.getPacman();
		List<Monster> monsters = tray.getMonsters();
		AnimationController animador = new AnimationController(monsters.size());			
		
		tray.setImage( MultimidiaFacade.getInstance().getImagemTabuleiro(fase.getNumber()) ); 			
		pacman.setImage( MultimidiaFacade.getInstance().getImagemPersonagem(pacman, animador) );			
		for(Monster monster:monsters) 
			monster.setImage( MultimidiaFacade.getInstance().getImagemPersonagem(monster, animador) );
		
		fase.setAnimation(animador);
		
		algoritmoPacman = new PacmanAlgorithm(game, pacman, fase);
		algoritmosMonster = new MonsterAlgorithm[monsters.size()];
		for(int i = 0; i < algoritmosMonster.length; i++) {
			Monster monster = monsters.get(i);
			algoritmosMonster[i] = new MonsterAlgorithm(game, monster, fase);
		}	
		algoritmoPicture = new PictureAlgorithm(game, fase);
	}
			
	private class AnimationController implements Animation {

		public final static int QUANTIDADE = 3;
		
		private int contadorComeCome = 1;
		private int[] contadoresMonstrinho;  
		
		public AnimationController(int nMonstrinhos) {
			contadoresMonstrinho = new int[nMonstrinhos];
			for(int i = 0; i < contadoresMonstrinho.length; i++)
				contadoresMonstrinho[i] = 1;
		}
				
		public String getPacmanPath(String direcao) {
			ImagemPersonagem imagemPersonagem = new ImagemComeCome(String.valueOf(contadorComeCome));
			Midia midia = new ImagemDirecao(imagemPersonagem, direcao);
			contadorComeCome = (contadorComeCome%QUANTIDADE)+1;
			return midia.getCaminho();
		}
		
		public String getMonsterPath(String id, String estado) {
			int indice = Integer.parseInt(id) - 1;
			ImagemPersonagem imagemPersonagem = new ImagemMonstrinho(String.valueOf(contadoresMonstrinho[indice]));
			ImagemPersonagem imagemEstado = new ImagemEstado(imagemPersonagem, estado);
			String caminho;
			if(estado.equals(StateConstants.STRING_STRONG_STATE)) {
				ImagemPersonagem imagemID = new ImagemID(imagemEstado, id);
				caminho = imagemID.getCaminho();
			} else {
				caminho = imagemEstado.getCaminho();
			}			
			contadoresMonstrinho[indice] = (contadoresMonstrinho[indice]%QUANTIDADE)+1;
			return caminho;
		}
				
	}
			
}
