package italo.pacman.controller.algorithm;

import italo.pacman.algorithm.AbstractAlgorithm;
import italo.pacman.algorithm.time.TimeAlgorithm;
import italo.pacman.algorithm.time.TimeConditions;
import italo.pacman.constants.StateConstants;
import italo.pacman.logic.game.Fase;
import italo.pacman.logic.game.Monster;

public class TimeStateAlgorithm extends AbstractAlgorithm implements TimeConditions {

	private Monster monster;
	private Fase fase;
	private int time;
		
	private boolean executed;
	
	public TimeStateAlgorithm(Monster monster, Fase fase, int time) {
		super("THREAD (TIME STATE)");
		this.monster = monster;
		this.fase = fase;
		this.time = time;
		this.executed = false;
	}
	
	public void timeBegined() {
		
	}
	
	public void timeEnded() {
		
	}
	
	public void begined() {
		
	}
	
	public void ended() {		
			
	}
	
	public boolean pauseCondition() {
		return fase.getGame().isPause();
	}
	
	public boolean executionCondition() {
		return !executed && !fase.finalized() && !monster.getState().finalizeTimeStateThread();
	}
	
	public boolean pausable() {
		return true;
	}
			
	public void execute() {		
		TimeAlgorithm timeAlgorithm = new TimeAlgorithm(this, time);
		timeAlgorithm.start();
		timeAlgorithm.join();
		if(executionCondition()) {
			monster.setState(StateConstants.MIDDLE_WEAK_STATE);
			timeAlgorithm.start();
			timeAlgorithm.join();
			if(executionCondition())
				monster.setState(StateConstants.STRONG_STATE);
		}
		this.executed = true;
	}

}
