package italo.pacman.load.fase;

import italo.pacman.load.attributes.LoadFaseAttributes;
import italo.pacman.load.attributes.LoadMonsterAttributes;
import italo.pacman.load.attributes.LoadPacmanAttributes;
import italo.pacman.load.attributes.LoadTrayAttributes;
import italo.pacman.load.fase.properties.FaseProperties;
import italo.pacman.logic.game.attributes.FaseAttributes;
import italo.pacman.logic.game.attributes.MonsterAttributes;
import italo.pacman.logic.game.attributes.PacmanAttributes;
import italo.pacman.logic.game.attributes.TrayAttributes;

public class FaseLoad {
	
	private FaseProperties faseProperties;
	
	public FaseLoad(FaseProperties faseProperties) {
		this.faseProperties = faseProperties;
	}
	
	public FaseAttributes load() {
		return this.loadFaseAttributes();
	}

	private FaseAttributes loadFaseAttributes() {
		LoadFaseAttributes faseAttributes = new LoadFaseAttributes();
		faseAttributes.setTrayAttributes(this.loadTrayAttributes());
		return faseAttributes;
	}
			
	private TrayAttributes loadTrayAttributes() {
		LoadTrayAttributes trayAttributes = new LoadTrayAttributes();		
		trayAttributes.setInitialPersonageDimensionFactor(faseProperties.getInitialPersonageDimensionFactor());
		trayAttributes.setWidthWalls(faseProperties.getWidthWalls());
		trayAttributes.setHorizontalWalls(faseProperties.getHorizontalWalls());
		trayAttributes.setVerticalWalls(faseProperties.getVerticalWalls());
		trayAttributes.setBolls(faseProperties.getBolls());
		trayAttributes.setBigBolls(faseProperties.getBigBolls());
		trayAttributes.setHouseCellIndexes(faseProperties.getHouseCellIndexes());
		trayAttributes.setHorizontalHouseDoorIndexes(faseProperties.getHorizontalHouseDoorIndexes());
		trayAttributes.setVerticalHouseDoorIndexes(faseProperties.getVerticalHouseDoorIndexes());
		trayAttributes.setPacmanAttributes(this.getPacmanAttributes());
		trayAttributes.setMonstersAttributes(this.getMonstersAttributes());		
		return trayAttributes;
	}

	private PacmanAttributes getPacmanAttributes() {
		LoadPacmanAttributes pacmanAttributes = new LoadPacmanAttributes();		
		pacmanAttributes.setSpeed(faseProperties.getPacmanSpeed());
		pacmanAttributes.setInitialXPosition(faseProperties.getInitialPacmanPosition()[0]);
		pacmanAttributes.setInitialYPosition(faseProperties.getInitialPacmanPosition()[1]);		
		pacmanAttributes.setInitialDirection(faseProperties.getInitialPacmanDirection());
		return pacmanAttributes;
	}
	
	private MonsterAttributes[] getMonstersAttributes() {		
		int[] monsterSpeeds = faseProperties.getMonstersSpeed();
		int[][] initialMonsterPosictions = faseProperties.getInitialMonstersPosition();		
		String[] initialMonsterDirections = faseProperties.getInitialMonstersDirection();						
		String[] initialMonstersState = faseProperties.getInitialMonstersState();
		
		LoadMonsterAttributes[] monstersAttributes = new LoadMonsterAttributes[initialMonsterDirections.length];
		for(int i = 0; i < monstersAttributes.length; i++) {
			monstersAttributes[i] = new LoadMonsterAttributes();
			monstersAttributes[i].setId(String.valueOf(i+1));
			monstersAttributes[i].setSpeed(monsterSpeeds[i]);
			monstersAttributes[i].setState( initialMonstersState[i] );
			monstersAttributes[i].setInitialDirection(initialMonsterDirections[i]); 
			monstersAttributes[i].setInitialXPosition(initialMonsterPosictions[i][0]);
			monstersAttributes[i].setInitialYPosition(initialMonsterPosictions[i][1]);
		}		
		return monstersAttributes;
	}
		
}
