package italo.pacman.logic.abertura.vo;

import italo.pacman.adapter.figure.AbstractContainerFigure;
import italo.pacman.adapter.figure.abertura.ProgressBarFigure;
import italo.pacman.adapter.picture.abertura.LogicAberturaPicture;
import italo.pacman.logic.Sistema;
import italo.pacman.logic.abertura.Abertura;
import italo.pacman.logic.abertura.event.AberturaListener;
import italo.pacman.logic.abertura.mediator.AberturaMediator;
import italo.pacman.logic.game.ProgressBar;
import italo.pacman.logic.game.bound.Bound;
import italo.pacman.logic.game.vo.ProgressBarVO;

import java.awt.Image;

public class AberturaVO extends AbstractContainerFigure implements Abertura {
	
	private AberturaMediator mediator;
	private ProgressBar progressBar;
	private AberturaListener listener;

	private Sistema sistema;
		
	public Sistema getSistema() {
		return sistema;
	}

	public AberturaVO(Sistema sistema, int x, int y, int width, int height) {
		super(null, x, y, width, height);
		this.mediator = new AberturaMediator(this);
		this.progressBar = new ProgressBarVO(this);
		this.sistema = sistema;
	}
	
	public AberturaVO(Sistema sistema) {
		this(sistema, 0, 0, 0, 0);
	}

	public void setAberturaListener(AberturaListener listener) {
		this.listener = listener;
	}
	
	public void removeAberturaListener() {
		this.listener = null;
	}
	
	public void initialize() {
		if(listener != null)
			listener.initialize(this);
	}
	
	public void init() {
		if(listener != null)
			listener.init(this);
	}
	
	public void initialize(Bound bound) {
		setX( bound.getX() );
		setY( bound.getY() );
		setWidth( bound.getWidth() );
		setHeight( bound.getHeight() );
	}
	
	public void init(LogicAberturaPicture picture) {
		mediator.init(picture);
	}
	
	public void setImage(Image image) {
		mediator.setImage(image);
	}

	public void setPercentage(int percentage) {
		progressBar.setPercentage(percentage);
	}
	
	public ProgressBarFigure getProgressBarFigure() {
		return progressBar;
	}
	
	public ProgressBar getProgressBar() {
		return progressBar;
	}


}
