package italo.pacman.logic.game.astar.algoritmo;

import italo.pacman.logic.game.Personage;
import italo.pacman.logic.game.Tray;
import italo.pacman.logic.game.astar.AStar;
import italo.pacman.logic.game.astar.AStarCellPosition;
import italo.pacman.logic.game.astar.algoritmo.util.AStarUtil;
import italo.pacman.logic.game.astar.algoritmo.valueobject.AStarCellPositionVO;
import italo.pacman.logic.game.astar.algoritmo.valueobject.AStarCellVO;
import italo.pacman.logic.game.direction.Direction;

import java.util.LinkedList;
import java.util.List;

public class AStarAlgoritmo implements AStar {

	public final static int FATOR_VISINHANCA = 10;
	
	private Tray tray;
	private AStarCellPositionVO initialCellPosition;
	private AStarCellPositionVO targetCellPosition;
	
	private AStarCellVO[][] cells;
	private LinkedList<AStarCellPositionVO> openedList = new LinkedList<AStarCellPositionVO>();
	private LinkedList<AStarCellPositionVO> closedList = new LinkedList<AStarCellPositionVO>();
	
	private LinkedList<AStarCellPosition> optimunRouteCells = new LinkedList<AStarCellPosition>();
				
	public boolean execute(Tray tray, Personage monster, Personage pacman) {				
		int nCelulasH = tray.horizontalCellQuantity();
		int nCelulasV = tray.verticalCellQuantity();
		this.cells = new AStarCellVO[nCelulasH][nCelulasV];
		for(int i = 0; i < nCelulasH; i++)
			for(int j = 0; j < nCelulasV; j++) 
				cells[i][j] = new AStarCellVO(i, j);									
		int widthCell = tray.getCell().getWidth();
		int heightCell = tray.getCell().getHeight();		
		int xCellInitial = monster.getX() / widthCell;
		int yCellInitial = monster.getY() / heightCell;		
		int xCellAlvo = pacman.getX() / widthCell;
		int yCellAlvo = pacman.getY() / heightCell;				
		this.initialCellPosition = cells[xCellInitial][yCellInitial].getPosition();
		this.targetCellPosition  = cells[xCellAlvo][yCellAlvo].getPosition();		
		this.tray = tray;		
		return this.execute();
	}
	
	public LinkedList<AStarCellPosition> getOptimunRouteCells() {
		return optimunRouteCells;
	}
	
	private boolean execute() {	
		AStarCellPositionVO currentCellPosition = null;		
		openedList.add(initialCellPosition); 		
		while( !openedList.isEmpty() ) {	
			this.ordenaPorF( openedList );
			currentCellPosition = openedList.removeFirst();			
			closedList.add( currentCellPosition );
			if( this.isEqualPositions(targetCellPosition, currentCellPosition) )
				break;
			List<AStarCellPositionVO> vizinhas = getCelulasVizinhasSemObstaculo( currentCellPosition );				
			this.ordenaPorF( vizinhas );
			for( AStarCellPositionVO pVizinha:vizinhas ) {
				if( isInClosedList(pVizinha) )
					continue;	
				int xVizinha = pVizinha.getX();
				int yVizinha = pVizinha.getY();
				int xCurrent = currentCellPosition.getX();
				int yCurrent = currentCellPosition.getY();
				int distAlvo = (int) AStarUtil.distancia(pVizinha, targetCellPosition);
				int	gNovo = cells[xCurrent][yCurrent].getG() + FATOR_VISINHANCA; 
				int hNovo = distAlvo * FATOR_VISINHANCA;
				AStarCellVO cVizinha = cells[xVizinha][yVizinha];
				if( !this.isInOpenedList( pVizinha ) ) {	
					openedList.add( pVizinha );						
					cVizinha.setPositionParent( currentCellPosition );
					cVizinha.setG(gNovo);
					cVizinha.setH(hNovo);
				} else if( gNovo < cVizinha.getG() ) {						
					cVizinha.setPositionParent( currentCellPosition );
					cVizinha.setG(gNovo);
					cVizinha.setH(hNovo);						
				}
			}			
		}		
		if(!openedList.isEmpty()) {
			while( !this.isEqualPositions(initialCellPosition, currentCellPosition) ) {			
				optimunRouteCells.add(currentCellPosition);		
				currentCellPosition = cells[currentCellPosition.getX()][currentCellPosition.getY()].getPositionParent();				
			}
			return true;
		}
		return false;
	}
	
	private List<AStarCellPositionVO> getCelulasVizinhasSemObstaculo(AStarCellPositionVO position) {
		List<AStarCellPositionVO> celulasP = new LinkedList<AStarCellPositionVO>();
		int nCelulasH = tray.horizontalCellQuantity();
		int nCelulasV = tray.verticalCellQuantity();		
		Direction[] directions = Direction.DIRECTIONS;
		int[][] directionsXY_CELL = Direction.DIRECTIONS_XY_CELL;
		int[][] directionsXY_WALL = Direction.DIRECTIONS_XY_WALL;	
		for(int i = 0; i < directions.length; i++) {
			int xP = position.getX();
			int yP = position.getY();
			int xWall = xP + directionsXY_WALL[i][0];
			int yWall = yP + directionsXY_WALL[i][1]; 
			int xCell = xP + directionsXY_CELL[i][0];
			int yCell = yP + directionsXY_CELL[i][1];
			Direction direction = directions[i];
			if(xCell >= 0 && yCell >= 0 && xCell < nCelulasH && yCell < nCelulasV) {
				if(direction.isFreeWall(xWall, yWall, tray)) { 
					celulasP.add(new AStarCellPositionVO(xCell, yCell)); 
				}
			}
		}
		return celulasP;		
	}
	
	private boolean isInOpenedList(AStarCellPositionVO position) {
		for(AStarCellPositionVO p:openedList) 
			if(isEqualPositions(position, p)) 
				return true;
		return false; 
	}
	
	private boolean isInClosedList(AStarCellPositionVO position) {
		for(AStarCellPositionVO p:closedList) 
			if(isEqualPositions(position, p)) 
				return true;
		return false;
	}
		
	private boolean isEqualPositions(AStarCellPositionVO pC1, AStarCellPositionVO pC2) {
		if(pC1 != null && pC2 != null) {
			return (pC1.getX() == pC2.getX() && pC1.getY() == pC2.getY());
		} else if(pC1 == null && pC2 == null) {
			throw new RuntimeException("Erro: Posicao nula.");
		} else {
			return false;
		}		       
	}
	
	private void ordenaPorF(List<AStarCellPositionVO> positions) {
		boolean ordenado;
		do {
			ordenado = true;
			for(int i = 0; i < (positions.size()-1); i++) {
				AStarCellPositionVO p1 = positions.get(i);
				AStarCellPositionVO p2 = positions.get(i+1);
				AStarCellVO c1 = cells[p1.getX()][p1.getY()];
				AStarCellVO c2 = cells[p2.getX()][p2.getY()];
				if(c1.getF() > c2.getF()) {
					positions.set(i, p2);
					positions.set(i+1, p1);
					ordenado = false;
				}
			}
		} while(!ordenado);
	}
	
}
