package italo.pacman.logic.game.direction;

import italo.pacman.logic.game.Personage;
import italo.pacman.logic.game.Tray;

public class DownDirection implements Direction {

	public int getXNew(Personage personagem, Tray tabuleiro) {
		return personagem.getX();
	}
	
	public int getYNew(Personage personagem, Tray tabuleiro) {
		int yP = personagem.getY();
		int yParede = getYWall(personagem, tabuleiro);
		if(tabuleiro.isLateralHorizontalWall(yParede))
			return 0;
		return yP+VALOR_INCREMENTO;
	}
	
	public int getXWall(Personage personagem, Tray tabuleiro) {
		int xP = personagem.getX();
		int larguraC = tabuleiro.getCell().getWidth();
		return xP/larguraC;
	}
	
	public int getYWall(Personage personagem, Tray tabuleiro) {
		int yP = personagem.getY();
		int alturaC = tabuleiro.getCell().getHeight();
		return (yP+alturaC)/alturaC;
	}

	public boolean isFreeWall(Personage personagem, Tray tabuleiro) {
		int xParede = getXWall(personagem, tabuleiro);
		int yParede = getYWall(personagem, tabuleiro);
		if(personagem.crossWall()) {
			if(!tabuleiro.isLateralHorizontalWall(yParede)) 
				return true;
		}			
		return tabuleiro.freeHorizontalWall(xParede, yParede);
	}
	
	public boolean isFreeWall(int xParede, int yParede, Tray tabuleiro) {
		return tabuleiro.freeHorizontalWall(xParede, yParede);
	}
	
	public boolean validMotion(Personage personagem, Tray tabuleiro) {
		int xP = personagem.getX();
		int larguraC = tabuleiro.getCell().getWidth();
		return (xP % larguraC == 0);
	}
	
	public boolean posicaoValidaParaComer(Personage personagem, Tray tabuleiro) {
		int yP = personagem.getY();
		int meiaAC = tabuleiro.getCell().getHeight()/2; 
		return (yP % meiaAC == 0);
	}
	
	public boolean isHouseDoor(Personage personagem, Tray tabuleiro) {
		int xParede = getXWall(personagem, tabuleiro);
		int yParede = getYWall(personagem, tabuleiro);
		return tabuleiro.isHorizontalDoor(xParede, yParede);
	}
	
	public Direction[] getRemainderDirection() {
		return new Direction[] {LEFT_DIRECTION, RIGHT_DIRECTION, UP_DIRECTION};
	}
	
	public Direction getOppositeDirection() {
		return UP_DIRECTION;
	}
	
	public String getString() {
		return STRING_DOWN_DIRECTION;
	}
	
	public String toString() {
		return getString();
	}
	
}
