package italo.pacman.logic.game.mediator;

import italo.pacman.adapter.picture.game.LogicGamePicture;
import italo.pacman.constants.GameConstants;
import italo.pacman.logic.AbstractFigureMediator;
import italo.pacman.logic.game.Fase;
import italo.pacman.logic.game.Game;
import italo.pacman.logic.game.attributes.FaseAttributes;
import italo.pacman.logic.game.attributes.GameAttributes;
import italo.pacman.logic.game.difficulty.DifficultyUtil;
import italo.pacman.logic.game.vo.FaseVO;


public class GameMediator extends AbstractFigureMediator {

	private int initialLifeQuantity;	
	private FaseController faseController;
	
	private Game game;
	
	private boolean endGame;
	private boolean pause;
	
	public GameMediator(Game game) {
		this.game = game;
		this.endGame = false;
		this.pause = false;
	}
				
	public void initialize(GameAttributes gameAttributes) {
		addFases( gameAttributes.getFasesAttributes() );		
		game.setDifficulty(DifficultyUtil.getDificuldade(gameAttributes.getDifficulty()));
		game.getScore().setScoreValue( GameConstants.INITIAL_SCORE );
		this.initialLifeQuantity = gameAttributes.getLifeQuantity();
		this.faseController = new FaseController( game.getFases() );
		this.updateComponentList();
	}
		
	public void init(LogicGamePicture picture) {
		picture.init(game);
		super.setLogicPicture(picture);		
		this.endGame = false;
		this.pause = false;
		game.setLifeQuantity(initialLifeQuantity);
		game.getScore().init( picture.getScorePicture() );
	}
	
	public void finish() {
		if(pause) {
			pause = false;
			game.continueGame();		
		}
		endGame = true;		
	}
	
	public boolean isEndGame() {
		return endGame;
	}
	
	public void gamePause() {
		if(!game.finalized()) {
			pause = ! pause;
			if(pause) {
				game.pauseGame();			
			} else {
				game.continueGame();			
			}
		}
	}
	
	private void updateComponentList() {
		game.add( game.getScore() );
		game.add( game.getCurrentFase().getTray() );
	}

	private void addFases(FaseAttributes[] atributosFases) { 
		for(int i = 0; i < atributosFases.length; i++) {
			FaseAttributes atributosFase = atributosFases[i];
			Fase fase = new FaseVO(game);
			fase.setNumber(String.valueOf(i+1)); 
			fase.initialize( atributosFase );
			addFase( fase );			
		}	
	}
			
	public boolean addFase(Fase fase) {
		return game.getFases().add(fase);
	}
	
	public boolean removeFase(Fase fase) {
		return game.getFases().remove(fase);
	}
	
	public int getInitialLifeQuantity() {
		return initialLifeQuantity;
	}

	public FaseController getFaseController() {
		return faseController;
	}

	public void setPause(boolean pause) {			
		this.pause = pause;
	}
	
	public boolean isPause() {
		return pause;
	}

}
