package italo.pacman.logic.game.mediator;

import italo.pacman.adapter.picture.game.LogicPersonagePicture;
import italo.pacman.logic.AbstractFigureMediator;
import italo.pacman.logic.game.Personage;
import italo.pacman.logic.game.attributes.PersonageAttributes;
import italo.pacman.logic.game.direction.Direction;
import italo.pacman.logic.game.direction.DirectionUtil;


public class PersonageMediator extends AbstractFigureMediator { 

	protected int initialX;
	protected int initialY;
	protected Direction initialDirection;
	protected Direction newDirection;
	
	protected Personage personage;
		
	public PersonageMediator(Personage personage) {
		this.personage = personage;
	}
	
	public void initialize(PersonageAttributes personageAttributes) {
		this.initialX = personageAttributes.getInitialXPosition();
		this.initialY = personageAttributes.getInitialYPosition();
		this.initialDirection = DirectionUtil.getDirection(personageAttributes.getInitialDirection());
		this.personage.setSpeed(personageAttributes.getSpeed());
	}
	
	public void init(LogicPersonagePicture picture) {
		picture.init(personage);
		super.setLogicPicture(picture);
		personage.setX(initialX);
		personage.setY(initialY);
		personage.setDirection(initialDirection);
		newDirection = initialDirection;
	}
	
	public void reinit() {
		personage.setX(initialX);
		personage.setY(initialY);
		personage.setDirection(initialDirection);
		newDirection = initialDirection;	
	}
	
	public void calculateInitialXY(int largura, int altura) {
		initialX = initialX*largura;
		initialY = initialY*altura;
	}
	
	public void resiseXY(int oldWidth, int oldHeight, int width, int height) {
		double xFactor = ((double)personage.getX())/oldWidth;
		double yFactor = ((double)personage.getY())/oldHeight;
		personage.setX( ((int)(xFactor*width)) );
		personage.setY( ((int)(yFactor*height)) );
	}
									
	public int getInitialX() {
		return initialX;
	}

	public int getInitialY() {
		return initialY;
	}

	public Direction getInitialDirection() {
		return initialDirection;
	}
	
	public Direction getNewDirection() {
		return newDirection;
	}

	public void setInitialX(int initialX) {
		this.initialX = initialX;
	}

	public void setInitialY(int initialY) {
		this.initialY = initialY;
	}

	public void setInitialDirection(Direction initialDirection) {
		this.initialDirection = initialDirection;
	}
	
	public void setNewDirection(Direction newDirection) {
		this.newDirection = newDirection;
	}

}
