package italo.pacman.logic.game.mediator;

import italo.pacman.adapter.picture.game.LogicTrayPicture;
import italo.pacman.logic.AbstractFigureMediator;
import italo.pacman.logic.game.Monster;
import italo.pacman.logic.game.Pacman;
import italo.pacman.logic.game.Personage;
import italo.pacman.logic.game.Tray;
import italo.pacman.logic.game.attributes.MonsterAttributes;
import italo.pacman.logic.game.attributes.PacmanAttributes;
import italo.pacman.logic.game.attributes.TrayAttributes;
import italo.pacman.logic.game.bound.Bound;
import italo.pacman.logic.game.direction.Direction;
import italo.pacman.logic.game.vo.MonsterVO;
import italo.pacman.logic.game.vo.PacmanVO;
import italo.pacman.utilities.VectorUtil;


public class TrayMediator extends AbstractFigureMediator { 

	private Tray tray;
	
	private boolean[][] initialBolls;
	private int[][] initialBigBolls;
	
	private double initialPersonageDimensionFactor;
	
	public TrayMediator(Tray tabuleiro) {
		this.tray = tabuleiro;
	}
		
	public void initialize(TrayAttributes trayAttributes) {						
		this.initialPersonageDimensionFactor = trayAttributes.getInitialPersonageDimensionFactor();
		this.initialBolls = trayAttributes.getBolls();
		this.initialBigBolls = trayAttributes.getBigBolls();
		addPacman( trayAttributes.getPacmanAttributes() );
		addMonsters( trayAttributes.getMonstersAttributes() );		
		tray.setWidthWall( trayAttributes.getWidthWalls() );
		tray.setHorizontalWalls(trayAttributes.getHorizontalWalls());
		tray.setVerticalWalls(trayAttributes.getVerticalWalls());
		tray.getHouse().setCellIndexes(trayAttributes.getHouseCellIndexes());
		tray.getHouse().setHorizontalDoorIndexes(trayAttributes.getHorizontalHouseWallIndexes());
		tray.getHouse().setVerticalDoorIndexes(trayAttributes.getVerticalHouseWallIndexes());
		this.updateComponentList();
	}
	
	public void init(LogicTrayPicture picture) {
		picture.init(tray);
		super.setLogicPicture(picture);
		tray.getBoll().setBolls(VectorUtil.copy(initialBolls));
		tray.getBigBoll().setBigBolls(VectorUtil.copy(initialBigBolls));
		tray.getBoll().init( picture.getBollPicture() );
		tray.getBigBoll().init( picture.getBigBollPicture() );
		tray.getCell().init( picture.getCellPicture() );
		tray.getHouse().init( picture.getHousePicture() );
		tray.getMessage().init( picture.getMessagePicture() );
		tray.getPacman().init( picture.getPacmanPicture() );	
		for(int i = 0; i < tray.getMonsters().size(); i++) {
			Monster monster = tray.getMonsters().get(i);
			monster.init( picture.getMonstersPicture().get(i) ); 		
		}		
	}
			
	public void reinit() {
		tray.getPacman().reinit();
		for(Monster monster:tray.getMonsters())
			monster.reinit();		
	}
	
	private void updateComponentList() {				
		tray.add( tray.getCell() );
		tray.add( tray.getBoll() );
		tray.add( tray.getBigBoll() );
		tray.add( tray.getHouse() );
		tray.add( tray.getMessage() );
		tray.add( tray.getPacman() );
		for(Monster monster:tray.getMonsters())
			tray.add( monster );
	}
	
	private void addPacman (PacmanAttributes pacmanAttributes) {
		Pacman pacman = new PacmanVO(tray);
		pacman.initialize( pacmanAttributes );
		tray.setPacman(pacman);
	}
	
	private void addMonsters(MonsterAttributes[] monsterAttributes) {
		for(int i = 0; i < monsterAttributes.length; i++) {
			Monster monster = new MonsterVO(tray);
			monster.initialize( monsterAttributes[i] );
			tray.addMonster( monster );
		}
	}
	
	public void resize(Bound bound) {		
		int boundX = bound.getX();
		int boundY = bound.getY();
		int boundWidth = bound.getWidth();
		int boundHeight = bound.getHeight();
		
		int nCellH = this.horizontalCellQuantity();
		int nCellV = this.verticalCellQuantity();	
		int widthWall = tray.getWidthWall();
		int width  = ( ( ( boundWidth  - (widthWall) ) / nCellH ) * nCellH );
		int height = ( ( ( boundHeight - (widthWall) ) / nCellV ) * nCellV );
		int x = ( ( boundWidth  % width  ) / 2 ) + boundX;
		int y = ( ( boundHeight % height ) / 2 ) + boundY;
		int widthC  = width  / nCellH;
		int heightC = height / nCellV;
		tray.setX(x);
		tray.setY(y);
		tray.setWidth(width);
		tray.setHeight(height);
		tray.getCell().setWidth(widthC);
		tray.getCell().setHeight(heightC);
		tray.getPacman().calculateInitialXY(widthC, heightC);
		tray.getPacman().redimensionar((int)(widthC*initialPersonageDimensionFactor), (int)(heightC*initialPersonageDimensionFactor)); 
		for(Monster monster:tray.getMonsters()) {
			monster.calculateInitialXY(widthC, heightC);
			monster.redimensionar((int)(widthC*initialPersonageDimensionFactor), (int)(heightC*initialPersonageDimensionFactor));
		}
	}
	
	public int horizontalCellQuantity() {
		if(tray.getHorizontalWalls() != null && tray.getHorizontalWalls().length > 0)
			return tray.getHorizontalWalls()[0].length;
		return -1;
	}
	
	public int verticalCellQuantity() {
		if(tray.getVerticalWalls() != null)
			return tray.getVerticalWalls().length;
		return -1;
	}

	public int initialBollQuantity() {
		if(initialBolls != null) {
			int cont = 0;
			for(boolean[] bollsVector:initialBolls)
				for(boolean boll:bollsVector)
					if(boll)
						cont++;
			return cont;
		}
		return -1;
	}
	
	public int initialBigBollQuantity() {
		if(initialBigBolls != null)
			return initialBigBolls.length;
		return -1;
	}
	
	public int bollQuantity() {
		boolean[][] bolls = tray.getBoll().getBolls();
		if(bolls != null) {
			int cont = 0;
			for(boolean[] bollsVector:bolls)
				for(boolean boll:bollsVector)
					if(boll)
						cont++;
			return cont;
		}
		return -1;
	}
			
	public int bigBollQuantity() {
		boolean[][] bolls = tray.getBoll().getBolls();
		int[][] bigBolls = tray.getBigBoll().getBigBolls();
		if(bigBolls != null) {
			int cont = 0;
			for(int[] boll:bigBolls) {
				int x = boll[0];
				int y = boll[1];
				if(bolls[y][x])
					cont++;
			}	
			return cont;
		}
		return -1;
	}
	
	public boolean isLateralHorizontalWall(int y) {
		boolean[][] paredesHorizontal = tray.getHorizontalWalls();
		return (y == 0 || y == paredesHorizontal.length-1); 
	}
	
	public boolean isLateralVerticalWall(int x) {
		boolean[][] paredesVertical = tray.getVerticalWalls();
		return (x == 0 || x == paredesVertical[0].length-1);
	}
			
	public boolean freeHorizontalWall(int x, int y) {
		return tray.getHorizontalWalls()[y][x];		
	}
	
	public boolean freeVerticalWall(int x, int y) {
		return tray.getVerticalWalls()[y][x];		
	}
			
	public boolean isBigBoll(int x, int y) {
		int[][] indices = tray.getBigBoll().getBigBolls();
		return VectorUtil.containCoordinates(indices, x, y);
	}
	
	public boolean isHorizontalDoor(int x, int y) {
		int[][] indices = tray.getHouse().getHorizontalDoorIndexes();
		return VectorUtil.containCoordinates(indices, x, y);		
	}
	
	public boolean isVerticalDoor(int x, int y) {
		int[][] indices = tray.getHouse().getVerticalDoorIndexes();
		return VectorUtil.containCoordinates(indices, x, y);
	}
	
	public void move(Personage personagem, Direction direcao) {	
		int xNovo = direcao.getXNew(personagem, tray);
		int yNovo = direcao.getYNew(personagem, tray);
		personagem.setX(xNovo);
		personagem.setY(yNovo);							
	}
	
	public boolean hasBoll(int x, int y) {
		return tray.getBoll().getBolls()[y][x];
	}
	
	public void eat(int x, int y) {
		tray.getBoll().getBolls()[y][x] = false;
	}
	
	public int getXBoll(Pacman comecome) {
		int xP = comecome.getX();
		int larguraC = tray.getCell().getWidth();
		int meiaLC = larguraC/2;
		return xP / meiaLC;
	}
	
	public int getYBoll(Pacman comecome) {
		int yP = comecome.getY();
		int alturaC = tray.getCell().getHeight();
		int meiaAC = alturaC/2;		
		return yP / meiaAC;
	}	
	
	public int getXCell(Personage personagem) {
		int xP = personagem.getX();
		int larguraC = tray.getCell().getWidth();
		return xP / larguraC;
	}
	
	public int getYCell(Personage personagem) {
		int yP = personagem.getY();
		int alturaC = tray.getCell().getHeight();
		return yP / alturaC;
	}	
		
	public boolean isWithinCell(int x, int y) { 
		int larguraC = tray.getCell().getWidth();
		int alturaC = tray.getCell().getHeight();
		return (x % larguraC == 0 && y % alturaC == 0);
	}
	
	public boolean isHouseCell(int x, int y) {
		int[][] vetor = tray.getHouse().getCellIndexes();
		return VectorUtil.containCoordinates(vetor, x, y);
	}
			
	public boolean[][] getInitialBolls() {
		return initialBolls;
	}

	public int[][] getInitialBigBolls() {
		return initialBigBolls;
	}
	
}
