package italo.pacman.logic.game.state;

import italo.pacman.constants.DirectionConstants;
import italo.pacman.constants.MotionConstants;
import italo.pacman.logic.game.Monster;
import italo.pacman.logic.game.Pacman;
import italo.pacman.logic.game.Tray;
import italo.pacman.logic.game.difficulty.Difficulty;
import italo.pacman.logic.game.direction.Direction;
import italo.pacman.logic.game.motion.Motion;

public abstract class AbstractState implements State {
	
	public boolean moveToHouseWall(Monster monster, Tray tray) {
		Direction[] directions = DirectionConstants.DIRECTIONS;
		for(int i = 0; i < directions.length; i++) {
			Direction direction = directions[i];
			int xP = monster.getX();
			int yP = monster.getY();			
			if(tray.isWithinCell(xP, yP) && direction.isHouseDoor(monster, tray)) { 
				monster.setDirection(direction);
				tray.move(monster, direction);
				monster.exitOfHouse();
				return true;
			}			
		}		
		return false;
	}
	
	public boolean moveToWithoutFromHouse(Monster monster, Tray tabuleiro) {
		int xCelula = tabuleiro.getXCell(monster);
		int yCelula = tabuleiro.getYCell(monster);
		if(tabuleiro.isHouseCell(xCelula, yCelula))		
			return this.moveToHouseWall(monster, tabuleiro);		
		return false;
	}
			
	public boolean moveToWay(Monster monstrinho, Tray tabuleiro, Difficulty dificuldade) {
		Motion motion = MotionConstants.WAY_MOTION;
		return this.move(motion, monstrinho, tabuleiro, dificuldade);
	}
	
	public boolean moveToNear(Monster monstrinho, Tray tabuleiro, Difficulty dificuldade) {
		Motion motion = MotionConstants.NEAR_MOTION;
		return this.move(motion, monstrinho, tabuleiro, dificuldade);
	}
	
	public boolean moveRandomly(Monster monstrinho, Tray tabuleiro) {
		Motion motion = MotionConstants.RANDOM_MOTION;
		return motion.move(monstrinho, tabuleiro);
	}
			
	private boolean move(Motion motion, Monster monstrinho, Tray tabuleiro, Difficulty dificuldade) {
		Pacman comecome = tabuleiro.getPacman();
		if(dificuldade.personagesNear(comecome, monstrinho, tabuleiro)) {
			if(dificuldade.haveCollision(comecome, monstrinho, tabuleiro)) { 
				haveCollision(monstrinho, comecome);
				return false;
			} else {				
				return motion.move(monstrinho, tabuleiro);				
			}
		} else {
			return this.moveRandomly(monstrinho, tabuleiro);		
		}
	}
			
	public void haveCollision(Monster monstrinho, Pacman comecome) {
		
	}
	
}
