package italo.pacman.logic.game.vo;

import italo.pacman.adapter.picture.game.LogicTrayPicture;
import italo.pacman.logic.game.Fase;
import italo.pacman.logic.game.Game;
import italo.pacman.logic.game.Tray;
import italo.pacman.logic.game.attributes.FaseAttributes;
import italo.pacman.logic.game.event.FaseListener;
import italo.pacman.logic.game.mediator.FaseMediator;
import italo.pacman.multimidia.Animation;

public class FaseVO implements Fase {

	private String number;	
	
	private int bollsQuantity;
	private Tray tray;	 
		
	private Game game;
				
	private FaseListener listener;
	
	private FaseMediator mediator;
	
	public FaseVO(Game game) {
		this.game = game;
		this.mediator = new FaseMediator(this);
	}
	
	public void setFaseListener(FaseListener listener) {
		this.listener = listener;
	}
	
	public void removeFaseListener() {
		this.listener = null;
	}
		
	public void initialize(FaseAttributes attributes) {		
		mediator.initialize(attributes);		
	}
		
	public void init(LogicTrayPicture picture) {		
		mediator.init(picture);		
	}
	
	public void initialize() {
		if(listener != null)
			listener.initialize(this);
	}
	
	public void init() {
		if(listener != null)
			listener.init(this);
	}
	
	public void reinit() {
		mediator.reinit();
		if(listener != null)
			listener.reinit(this);
	}
	
	public void finish() {
		mediator.finish();
		if(listener != null)
			listener.finish(this);
	}
	
	public void begin() {
		if(listener != null)
			listener.begin(this);
	}
	
	public void end() {
		if(listener != null)
			listener.end(this);
	}
	
	public String getNumber() {
		return number;
	}

	public void setNumber(String number) {
		this.number = number;
	}
	
	public Tray getTray() {
		return tray;
	}

	public void setTray(Tray tray) {
		this.tray = tray;
	}

	public int getBollsQuantity() {
		return bollsQuantity;
	}

	public void setBollsQuantity(int bollsQuantity) {
		this.bollsQuantity = bollsQuantity;
	}

	public Game getGame() {
		return game;
	}
			
	public boolean finalized() {
		return mediator.isEndFase() || game.finalized();
	}
				
	public int getInitialBollsQuantity() {
		return mediator.getInitialBollsQuantity();
	}

	public void setInitialBollsQuantity(int initialBollsQuantity) {
		mediator.setInitialBollsQuantity(initialBollsQuantity);
	}

	public Animation getAnimation() {
		return mediator.getAnimation();
	}

	public void setAnimation(Animation animation) {
		mediator.setAnimation(animation);
	}
	
}
