package italo.pacman.logic.game.vo;

import italo.pacman.adapter.figure.AbstractContainerFigure;
import italo.pacman.adapter.figure.game.ScoreFigure;
import italo.pacman.adapter.figure.game.TrayFigure;
import italo.pacman.adapter.picture.game.LogicGamePicture;
import italo.pacman.logic.Sistema;
import italo.pacman.logic.game.Fase;
import italo.pacman.logic.game.Game;
import italo.pacman.logic.game.Score;
import italo.pacman.logic.game.attributes.GameAttributes;
import italo.pacman.logic.game.difficulty.Difficulty;
import italo.pacman.logic.game.event.GameListener;
import italo.pacman.logic.game.mediator.GameMediator;

import java.awt.Image;
import java.util.ArrayList;
import java.util.List;

public class GameVO extends AbstractContainerFigure implements Game {
	
	private int lifeQuantity;
	private Difficulty difficulty;
	private boolean crossWall;
	private Score score;
	private List<Fase> fases = new ArrayList<Fase>();
			
	private GameListener listener;
			
	private GameMediator mediator;
	private Sistema sistema;
	
	public GameVO(Sistema sistema, int x, int y, int width, int height, boolean crossWall) {
		super(null, x, y, width, height);
		this.sistema = sistema;
		this.crossWall = crossWall;
		this.mediator = new GameMediator(this);
		this.score = new ScoreVO(this);
	}
			
	public GameVO(Sistema sistema, int x, int y, int width, int height) {
		this(sistema,x, y, width, height, false);
	}
	
	public GameVO(Sistema sistema, boolean crossWall) {
		this(sistema,0, 0, 0, 0, crossWall);
	}
	
	public GameVO(Sistema sistema) {
		this(sistema, false);
	}
					
	public void setGameListener(GameListener listener) {
		this.listener = listener;
	}
	
	public void removeGameListener() {
		this.listener = null;
	}
			
	public void initialize(GameAttributes gameAttributes) {
		mediator.initialize(gameAttributes);		
	}
		
	public void init(LogicGamePicture picture) {		
		mediator.init(picture);			
	}
	
	public void initialize() {
		if(listener != null)
			listener.initialize(this);		
	}
	
	public void init() {
		if(listener != null)
			listener.init(this);		
	}
				
	public void play() {
		if(listener != null)
			listener.play(this);
	}
		
	public void finish() {
		mediator.finish();
		if(listener != null)
			listener.finish(this);	
	}
	
	public void begin() {
		if(listener != null)
			listener.begin(this);
	}
	
	public void end() {
		if(listener != null)
			listener.end(this);
	}
	
	public void perdeu() {
		if(listener != null)
			listener.perdeu(this);
	}
	
	public void passouDeFase() {
		if(listener != null)
			listener.passouDeFase(this);
	}
	
	public void zerou() {
		if(listener != null)
			listener.zerou(this);
	}
	
	public void gameover() {
		if(listener != null)
			listener.gameover(this);
	}
	
	public void pauseGame() {
		if(listener != null)
			listener.pauseGame(this);
	}
	
	public void continueGame() {
		if(listener != null)
			listener.continueGame(this);
	}
	
	public int getLifeQuantity() {
		return lifeQuantity;
	}

	public void setLifeQuantity(int numeroDeVidas) {
		this.lifeQuantity = numeroDeVidas;
	}

	public Difficulty getDifficulty() {
		return difficulty;
	}

	public void setDifficulty(Difficulty dificuldade) {
		this.difficulty = dificuldade;
	}

	public List<Fase> getFases() {
		return fases;
	}

	public boolean isCrossWall() {
		return crossWall;
	}

	public void setCrossWall(boolean crossWall) {
		this.crossWall = crossWall;
	}
			
	public void setImage(Image image) {
		mediator.setImage(image);
	}
	
	public boolean finalized() {
		return mediator.isEndGame() || sistema.finalized();
	}
				
	public boolean addFase(Fase fase) {
		return mediator.addFase(fase);
	}
	
	public boolean removeFase(Fase fase) {
		return mediator.removeFase(fase);
	}
	
	public int getInitialLifeQuantity() {
		return mediator.getInitialLifeQuantity();
	}

	public void setPause(boolean pause) {			
		mediator.setPause(pause);
	}
	
	public boolean isPause() {
		return mediator.isPause();
	}
	
	public Fase getCurrentFase() {
		return mediator.getFaseController().getCurrentFase();
	}
	
	public boolean isPauseGame() {
		return mediator.isPause();
	}
	
	public boolean gotoFase(int index) {
		return mediator.getFaseController().gotoFase(index);
	}
	
	public void firstFase() {
		mediator.getFaseController().firstFase();
	}
	
	public void nextFase() {
		mediator.getFaseController().nextFase();
	}
	
	public boolean isLastFase() {
		return mediator.getFaseController().isLastFase();
	}
	
	public int getFasesQuantity() {
		return mediator.getFaseController().getFases().size();
	}
	
	public int getCurrentFaseIndex() {
		return mediator.getFaseController().getCurrentFaseIndex();
	}
	
	public void gamePause() {
		mediator.gamePause();
	}

	public Score getScore() {
		return score;
	}

	public void setScore(Score score) {
		this.score = score;
	}

	public ScoreFigure getCurrentScoreFigure() {
		return score;
	}

	public TrayFigure getCurrentTrayFigure() {
		return mediator.getFaseController().getCurrentFase().getTray();
	}
	
	public Sistema getSistema() {
		return sistema;
	}
			
}
