package italo.pacman.logic.game.vo;

import italo.pacman.adapter.picture.game.LogicMonsterPicture;
import italo.pacman.logic.game.Monster;
import italo.pacman.logic.game.Pacman;
import italo.pacman.logic.game.Tray;
import italo.pacman.logic.game.astar.AStarCellPosition;
import italo.pacman.logic.game.attributes.MonsterAttributes;
import italo.pacman.logic.game.event.MonsterListener;
import italo.pacman.logic.game.mediator.MonsterMediator;
import italo.pacman.logic.game.state.State;

import java.util.List;


public class MonsterVO extends PersonageVO implements Monster {
	
	private State state;
	private List<AStarCellPosition> optimalRouteCells;
	private boolean routeFounded;
	
	private MonsterListener listener;
	
	public MonsterVO(Tray tray, String id, int sleep) {
		super(0, 0, 0, 0, tray, MONSTER, id, sleep);
		super.mediator = new MonsterMediator(this);
	}
	
	public MonsterVO(Tray tray) {
		this(tray, null, 0);
	}	
	
	public void setMonsterListener(MonsterListener listener) {
		this.listener = listener;
	}
		
	public void removeMonsterListener() {
		this.listener = null;
	}
	
	public void catchPacman(Pacman pacman) {
		if(listener != null)
			listener.catchPacman(this, pacman);
	}
	
	public void enterInHouse() {
		if(listener != null)
			listener.enterInHouse(this);
	}
	
	public void exitOfHouse() {
		if(listener != null)
			listener.exitOfHouse(this);
	}
	
	public void changeState(State estado) {
		if(listener != null)
			listener.changeState(this, estado);
	}
	
	public synchronized State getState() {
		return state;
	}

	public synchronized void setState(State state) {
		this.state = state;
		changeState(state);
	}
			
	public void initialize(MonsterAttributes attributes) {
		((MonsterMediator)mediator).initialize(attributes);
	}
		
	public void init(LogicMonsterPicture picture) {
		((MonsterMediator)mediator).init(picture);
	}
	
	public boolean canCrossWall() {
		return MONSTER_CROSS_WALL;
	}
	
	public boolean crossWall() {
		return canCrossWall() && tray.getFase().getGame().isCrossWall();
	}

	public List<AStarCellPosition> getOptimunRouteCells() {
		return optimalRouteCells;
	}
	
	public void setOptimunRouteCells(List<AStarCellPosition> optimalRouteCells) {
		this.optimalRouteCells = optimalRouteCells;
	}

	public boolean isRouteVisible() {
		return routeFounded;
	}

	public void setRouteVisible(boolean routeFounded) {
		this.routeFounded = routeFounded;
	}
	
}
