package italo.pacman.logic.game.vo;

import italo.pacman.adapter.figure.AbstractComponentFigure;
import italo.pacman.adapter.figure.game.TrayFigure;
import italo.pacman.adapter.picture.game.LogicPersonagePicture;
import italo.pacman.constants.PersonageConstants;
import italo.pacman.logic.game.Personage;
import italo.pacman.logic.game.Tray;
import italo.pacman.logic.game.direction.Direction;
import italo.pacman.logic.game.event.PersonageListener;
import italo.pacman.logic.game.mediator.PersonageMediator;

import java.awt.Image;

public abstract class PersonageVO extends AbstractComponentFigure implements Personage, PersonageConstants {
	
	protected String type;
	protected String id;	
	protected int speed;
	protected Direction direction;
		
	protected Tray tray;
	
	private PersonageListener listener;
	
	protected PersonageMediator mediator;
			
	protected PersonageVO(int x, int y, int width, int height, Tray tray, String tipo, String id, int velocidade) {
		super(tray, x, y, width, height);
		this.tray = tray;
		this.type = tipo;
		this.id = id;
		this.speed = velocidade;
	}
		
	public void setImage(Image image) {
		mediator.setImage(image);
	}
	
	public void setPersonageListener(PersonageListener listener) {
		this.listener = listener;
	}
	
	public void removePersonageListener() {
		this.listener = null;
	}
	
	public void repaint() {
		if(listener != null)
			listener.repaint(this);
	}
	
	public void init(LogicPersonagePicture picture) {
		if(listener != null)
			listener.init(this);
		mediator.init(picture);
	}
	
	public void reinit() {
		mediator.reinit();
		if(listener != null)
			listener.reinit(this);
	}
	
	
	public void finish() {
		if(listener != null)
			listener.finish(this);
	}
			
	public void initialize() {
		if(listener != null)
			listener.initialize(this);
	}
	
	public void init() {
		if(listener != null)
			listener.init(this);
	}
	
	public void begin() {
		if(listener != null)
			listener.begin(this);
	}
	
	public void end() {
		if(listener != null)
			listener.end(this);
	}
	
	public void move() {
		if(listener != null)
			listener.move(this);
	}
		
	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	public int getSpeed() {
		return speed;
	}

	public void setSpeed(int speed) {
		this.speed = speed;
	}

	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}
	
	public Direction getDirection() {
		return direction;
	}

	public void setDirection(Direction direction) {
		this.direction = direction;
	}

	public Tray getTray() {
		return tray;
	}
			
	public void calculateInitialXY(int largura, int altura) {
		mediator.calculateInitialXY(largura, altura);
	}
	
	public void resiseXY(int oudWidth, int oudHeight, int width, int height) {
		mediator.resiseXY(oudWidth, oudHeight, width, height);
	}
									
	public int getInitialX() {
		return mediator.getInitialX();
	}

	public int getInitialY() {
		return mediator.getInitialY();
	}

	public Direction getInitialDirection() {
		return mediator.getInitialDirection();
	}
	
	public Direction getNewDirection() {
		return mediator.getNewDirection();
	}

	public void setInitialX(int initialX) {
		mediator.setInitialX(initialX);
	}

	public void setInitialY(int initialY) {
		mediator.setInitialY(initialY);
	}

	public void setInitialDirection(Direction initialDirection) {
		mediator.setInitialDirection(initialDirection);
	}
	
	public void setNewDirection(Direction newDirection) {
		mediator.setNewDirection(newDirection);
	}
	
	public TrayFigure getTrayFigure() {
		return tray;
	}
	
}
