package italo.pacman.logic.game.vo;

import italo.pacman.adapter.figure.AbstractComponentFigure;
import italo.pacman.adapter.picture.game.LogicScorePicture;
import italo.pacman.logic.game.Game;
import italo.pacman.logic.game.Score;
import italo.pacman.logic.game.bound.Bound;
import italo.pacman.logic.game.event.ScoreListener;
import italo.pacman.logic.game.mediator.ScoreMediator;

import java.awt.Image;

public class ScoreVO extends AbstractComponentFigure implements Score {

	private int scoreValue;
	
	private ScoreMediator mediator;
	private Game game;
	private ScoreListener listener;
		
	public ScoreVO(Game game, int x, int y, int width, int height) {
		super(game, x, y, width, height);
		this.mediator = new ScoreMediator(this);
		this.game = game;
	}
	
	public ScoreVO(Game game) {		
		this(game, 0, 0, 0, 0);
	}
	
	public void setScoreListener(ScoreListener listener) {
		this.listener = listener;
	}
	
	public void removeScoreListener() {
		this.listener = null;
	}
	
	public void resize() {
		if(listener != null)
			listener.resize(this);
	}
		
	public void initialize(Bound bound) {
		mediator.resize(bound);
	}
		
	public void init(LogicScorePicture picture) {
		mediator.init(picture);
	}
	
	public void setImage(Image image) {
		mediator.setImage(image);
	}
	
	public int getLifeQuantity() {
		return game.getLifeQuantity();
	}

	public int getRemainderBolls() {
		return game.getCurrentFase().getBollsQuantity();
	}

	public int getFaseNumber() {
		return game.getCurrentFaseIndex()+1;
	}
	
	public void resize(Bound bound) {
		mediator.resize(bound);
	}

	public Game getGame() {
		return game;
	}

	public int getScoreValue() {
		return scoreValue;
	}

	public void setScoreValue(int scoreValue) {
		this.scoreValue = scoreValue;
	}
	
}
