package italo.pacman.logic.game.vo;

import italo.pacman.adapter.figure.AbstractContainerFigure;
import italo.pacman.adapter.figure.game.BigBollFigure;
import italo.pacman.adapter.figure.game.BollFigure;
import italo.pacman.adapter.figure.game.CellFigure;
import italo.pacman.adapter.figure.game.HouseFigure;
import italo.pacman.adapter.figure.game.MessageFigure;
import italo.pacman.adapter.figure.game.MonsterFigure;
import italo.pacman.adapter.figure.game.PacmanFigure;
import italo.pacman.adapter.picture.game.LogicTrayPicture;
import italo.pacman.logic.game.BigBoll;
import italo.pacman.logic.game.Boll;
import italo.pacman.logic.game.Cell;
import italo.pacman.logic.game.Fase;
import italo.pacman.logic.game.House;
import italo.pacman.logic.game.Message;
import italo.pacman.logic.game.Monster;
import italo.pacman.logic.game.Pacman;
import italo.pacman.logic.game.Personage;
import italo.pacman.logic.game.Tray;
import italo.pacman.logic.game.attributes.TrayAttributes;
import italo.pacman.logic.game.bound.Bound;
import italo.pacman.logic.game.direction.Direction;
import italo.pacman.logic.game.event.TrayListener;
import italo.pacman.logic.game.mediator.TrayMediator;

import java.awt.Image;
import java.util.ArrayList;
import java.util.List;

public class TrayVO extends AbstractContainerFigure implements Tray {
	
	private int widthWall;
	 
	private boolean[][] horizontalWalls;
	private boolean[][] verticalWalls;	
	private int[][] doorIndexes;
		
	private Cell cell;
	private House house;
	private Boll boll;
	private BigBoll bigBoll;
	
	private Pacman pacman;
	private List<Monster> monsters = new ArrayList<Monster>();
	
	private Message message;
	
	private Fase fase;		
	private TrayMediator mediator;
	
	private TrayListener listener;

	
	public TrayVO(Fase fase, int x, int y, int width, int height) {
		super(fase.getGame(), x, y, width, height);
		this.fase = fase;
		this.cell = new CellVO(this);
		this.boll = new BollVO(this);
		this.bigBoll = new BigBollVO(this);
		this.house = new HouseVO(this);
		this.message = new MessageVO(this);
		this.mediator = new TrayMediator(this); 
	}
			
	public TrayVO(Fase fase) {
		this(fase, 0, 0, 0, 0); 
	}

	public void setTrayListener(TrayListener listener) {
		this.listener = listener;
	}
	
	public void removeTrayListener() {
		this.listener = null;
	}
	
	public void resize() {
		if(listener != null)
			listener.resize(this);
	}
	
	public void initialize(TrayAttributes trayAttributes) {
		mediator.initialize(trayAttributes);
	}
	
	public void init(LogicTrayPicture trayPicture) {
		mediator.init(trayPicture);
	}
	
	public void reinit() {
		mediator.reinit();
	}
	
	public Fase getFase() {
		return fase;
	}
		
	public void setImage(Image image) {
		mediator.setImage(image);
	}
	
	public void resize(Bound bound) {		
		mediator.resize(bound);
	}
	
	public int horizontalCellQuantity() {
		return mediator.horizontalCellQuantity();
	}
	
	public int verticalCellQuantity() {
		return mediator.verticalCellQuantity();
	}

	public int initialBollQuantity() {
		return mediator.initialBollQuantity();
	}
	
	public int initialBigBollQuantity() {
		return mediator.initialBigBollQuantity();
	}
	
	public int bollQuantity() {
		return mediator.bollQuantity();
	}
			
	public int bigBollQuantity() {
		return mediator.bigBollQuantity();
	}
	
	public boolean isLateralHorizontalWall(int y) {
		return mediator.isLateralHorizontalWall(y);
	}
	
	public boolean isLateralVerticalWall(int x) {
		return mediator.isLateralVerticalWall(x);
	}
			
	public boolean freeHorizontalWall(int x, int y) {				
		return mediator.freeHorizontalWall(x, y);
	}
	
	public boolean freeVerticalWall(int x, int y) {
		return mediator.freeVerticalWall(x, y);
	}
			
	public boolean isBigBoll(int x, int y) {
		return mediator.isBigBoll(x, y);
	}
	
	public boolean isHorizontalDoor(int x, int y) {
		return mediator.isHorizontalDoor(x, y);	
	}
	
	public boolean isVerticalDoor(int x, int y) {
		return mediator.isVerticalDoor(x, y);
	}
	
	public void move(Personage personage, Direction direction) {	
		mediator.move(personage, direction);						
	}
	
	public boolean hasBoll(int x, int y) {
		return mediator.hasBoll(x, y);
	}
	
	public void eat(int x, int y) {
		mediator.eat(x, y);
	}
	
	public int getXBoll(Pacman pacman) {
		return mediator.getXBoll(pacman);
	}
	
	public int getYBoll(Pacman pacman) {
		return mediator.getYBoll(pacman);
	}	
	
	public int getXCell(Personage personagem) {
		return mediator.getXCell(personagem);
	}
	
	public int getYCell(Personage personagem) {
		return mediator.getYCell(personagem);
	}	
		
	public boolean isWithinCell(int x, int y) { 
		return mediator.isWithinCell(x, y);
	}
	
	public boolean isHouseCell(int x, int y) {
		return mediator.isHouseCell(x, y);
	}
			
	public boolean[][] getInitialBolls() {
		return mediator.getInitialBolls();
	}

	public int[][] getInitialBigBolls() {
		return mediator.getInitialBigBolls();
	}

	public int getWidthWall() {
		return widthWall;
	}

	public void setWidthWall(int widthWall) {
		this.widthWall = widthWall;
	}

	public boolean[][] getHorizontalWalls() {
		return horizontalWalls;
	}

	public void setHorizontalWalls(boolean[][] horizontalWalls) {
		this.horizontalWalls = horizontalWalls;
	}

	public boolean[][] getVerticalWalls() {
		return verticalWalls;
	}

	public void setVerticalWalls(boolean[][] verticalWalls) {
		this.verticalWalls = verticalWalls;
	}

	public int[][] getDoorIndexes() {
		return doorIndexes;
	}

	public void setDoorIndexes(int[][] doorIndexes) {
		this.doorIndexes = doorIndexes;
	}

	public Cell getCell() {
		return cell;
	}

	public void setCell(Cell cell) {
		this.cell = cell;
	}

	public House getHouse() {
		return house;
	}

	public void setHouse(House house) {
		this.house = house;
	}

	public Boll getBoll() {
		return boll;
	}

	public void setBoll(Boll boll) {
		this.boll = boll;
	}

	public BigBoll getBigBoll() {
		return bigBoll;
	}

	public void setBigBoll(BigBoll bigBoll) {
		this.bigBoll = bigBoll;
	}
	
	public Message getMessage() {
		return message;
	}

	public void setMessage(Message message) {
		this.message = message;
	}
	
	public Pacman getPacman() {
		return pacman;
	}

	public void setPacman(Pacman pacman) {
		this.pacman = pacman;
	}

	public List<Monster> getMonsters() {
		return monsters;
	}
	
	public boolean addMonster(Monster monster) {
		return monsters.add(monster);
	} 
	
	public boolean removeMonster(Monster monster) {
		return monsters.remove(monster);
	}

	public BigBollFigure getBigBollFigure() {
		return bigBoll;
	}

	public BollFigure getBollFigure() {
		return boll;
	}

	public CellFigure getCellFigure() {
		return cell;
	}
	
	public MessageFigure getMessageFigure() {
		return message;
	}

	public HouseFigure getHouseFigure() {
		return house;
	}

	public List<MonsterFigure> getMonstersFigure() {
		List<MonsterFigure> monstersList = new ArrayList<MonsterFigure>(); 
		monstersList.addAll(monsters);
		return monstersList;
	}

	public PacmanFigure getPacmanFigure() {
		return pacman;
	}
		
}
