package italo.pacman.logic.vo;

import italo.pacman.logic.Sistema;
import italo.pacman.logic.abertura.Abertura;
import italo.pacman.logic.abertura.vo.AberturaVO;
import italo.pacman.logic.encerramento.Encerramento;
import italo.pacman.logic.encerramento.vo.EncerramentoVO;
import italo.pacman.logic.event.SistemaListener;
import italo.pacman.logic.game.Game;
import italo.pacman.logic.game.vo.GameVO;
import italo.pacman.logic.mediator.SistemaMediator;
import italo.pacman.multimidia.MidiaLoad;


public class SistemaVO implements Sistema {
			
		
	private SistemaMediator mediator = new SistemaMediator(this);
	
	private Game game = new GameVO(this);
	private Abertura abertura = new AberturaVO(this);
	private Encerramento encerramento = new EncerramentoVO(this);
	
	private SistemaListener listener;
		
	public void setSistemaListener(SistemaListener listener) {
		this.listener = listener;
	}
			
	public void removeSistemaListener() {
		this.listener = null;
	}
	
	public void abertura() {
		if(listener != null)
			listener.abertura(this);
	}
	
	public void encerramento() {
		if(listener != null)
			listener.encerramento(this);
	}
	
	public void load() {
		if(listener != null)
			listener.load(this);
	}
			
	public void load(int nFiles) {
		mediator.load(nFiles);		
	}
	
	public void load(MidiaLoad midiaLoad) {
		if(listener != null)
			listener.load(this, midiaLoad);
	}
	
	public void loading(int percentage) {
		if(listener != null)
			listener.loading(this, percentage);
	}
	
	public void play() {
		if(listener != null)
			listener.play(this);
	}

	public void finish() {
		mediator.finish();
		if(listener != null)
			listener.finish(this);
	}
	
	public void begin() {
		if(listener != null)
			listener.begin(this);
	}
	
	public void end() {
		if(listener != null)
			listener.end(this);
	}
		
	public boolean finalized() {
		return mediator.isEndSistema();
	}
						
	public Game getGame() {
		return game;
	}
	
	public Abertura getAbertura() {
		return abertura;
	}
	
	public Encerramento getEncerramento() {
		return encerramento;
	}
	
}
