package italo.pacman.multimidia;

import java.applet.AudioClip;
import java.util.HashMap;
import java.util.Map;

class AudioController {
	
	private static AudioController singleton = null; 
	
	private Map<String, AudioClip> clips = new HashMap<String, AudioClip>();	
	private boolean executeAudio = true;
	
	private MidiaResource midiaResource;
	
	private AudioController() {}
	
	public static AudioController getInstance() {
		if(singleton == null) {
			synchronized(AudioController.class) {
				singleton = new AudioController();
			}
		}
		return singleton;
	}
	
	public void initialize(MidiaResource fileResource) {
		this.midiaResource = fileResource;
	}
	
	public void load(String[] audiosPath, MidiaLoad load, boolean c) {
		this.clear(c);
		this.load(audiosPath, load);
	}
	
	public void load(String[] audiosPath, MidiaLoad load) {		
		if(audiosPath != null) {
			for(int i = 0; !midiaResource.end() && i < audiosPath.length; i++) {
				clips.put(audiosPath[i], novoAudioClip(audiosPath[i]));				
				load.midiaLoaded();
			}
		}
	}
	
	public void execute(AudioClip audioClip) {
		if(executeAudio && !midiaResource.end())
			audioClip.play();		
	}
	
	public AudioClip getAudioClip(String caminho) {
		return clips.get(caminho);
	}
	
	private AudioClip novoAudioClip(String audioPath) {
		return midiaResource.getAudioClip( midiaResource.resourceURL( audioPath ) );
	}
	
	private void clear(boolean c) {
		if(c)
			clips.clear();
	}

	public boolean isExecuteAudio() {
		return executeAudio;
	}

	public void setExecuteAudio(boolean executeAudio) {
		this.executeAudio = executeAudio;
	}
	
}
