package italo.pacman.multimidia;

import java.awt.Image;
import java.awt.MediaTracker;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JPanel;


class ImageController {	
	
	private static ImageController singleton = null; 
	
	private Map<String, Image> images = new HashMap<String, Image>();
	
	private ImageController() {}
	
	private MidiaResource midiaResource;
	
	public static ImageController getInstance() {
		if(singleton == null) {
			synchronized(ImageController.class) {
				singleton = new ImageController();
			}
		}
		return singleton;
	}
			
	public void initialize(MidiaResource fileResource) {
		this.midiaResource = fileResource;
	}
	
	public void load(String imagePath, boolean c) {
		this.clear(c);
		this.load(imagePath);
	}
	
	public void load(String[] imagesPath, MidiaLoad load, boolean c) {
		this.clear(c);
		this.load(imagesPath, load);
	}
	
	public void load(String imagePath) {		
		MediaTracker tracker = new MediaTracker(new JPanel());
		imageLoad(imagePath, tracker, 0);
	}
	
	public void load(String[] imagesPath, MidiaLoad load) {
		if(imagesPath != null) {
			MediaTracker tracker = new MediaTracker(new JPanel());		
			for(int i = 0; !midiaResource.end() && i < imagesPath.length; i++) {		
				String caminho = imagesPath[i];
				imageLoad(caminho, tracker, i);
				load.midiaLoaded();
			}	
		}
	}

	public Image getImage(String imagePath) {
		return images.get(imagePath);
	}
				
	private void imageLoad(String imagePath, MediaTracker tracker, int id) {		
		URL url = midiaResource.resourceURL( imagePath );
		Image img = midiaResource.getImage(url);
		if(img != null && !images.containsValue(imagePath) && !midiaResource.end()) {	 
			tracker.addImage(img, id);
			try {
				tracker.waitForID(id);
			} catch (InterruptedException e) {
				
			}
			images.put(imagePath, img);
		}	
	}
	
	private void clear(boolean b) {
		if(b)
			images.clear();
	}
							
}
