package italo.pacman.multimidia.message;

import italo.pacman.algorithm.time.TimeAlgorithm;
import italo.pacman.algorithm.time.TimeConditions;
import italo.pacman.gui.GUIFacade;
import italo.pacman.logic.game.Game;
import italo.pacman.logic.game.Tray;

import java.awt.Image;


public class MessageController implements TimeConditions {
		
	private static MessageController singleton = null;
		
	private Game game;
	private boolean hide;
		
	private MessageController() {}
	
	public static MessageController getInstance() {
		if(singleton == null) {
			synchronized(MessageController.class) {
				singleton = new MessageController();
			}
		}
		return singleton;
	}
	
	public void desenharMensagem(Image image, Game game, double widthFactor, double heightFactor, int time, boolean hide, boolean prender) {
		this.game = game;
		this.hide = hide;
		Tray tray = game.getCurrentFase().getTray();
		tray.getMessage().setImage(image);
		tray.getMessage().resize(widthFactor, heightFactor);
		tray.getMessage().setVisible(true);
		GUIFacade.getInstance().repaint();
		
		TimeAlgorithm algorithm = new TimeAlgorithm("THREAD (MESSAGE)", this, time);
		algorithm.start();		
		if(prender)
			algorithm.join();
	}
					
	public void apagarMensagem() {
		game.getCurrentFase().getTray().getMessage().setVisible(false);
	}
	
	public void timeBegined() {
		
	}

	public void timeEnded() {
		Tray tray = game.getCurrentFase().getTray();
		tray.getMessage().setVisible(!hide);	
	}

	public boolean executionCondition() {
		Tray tray = game.getCurrentFase().getTray();
		return !game.finalized() && tray != null;
	}
	
	public boolean pauseCondition() {
		return false;
	}

	public boolean pausable() {
		return false;
	}
	
}
