package italo.pacman.picture.game.score;


import italo.pacman.adapter.figure.game.ScoreFigure;
import italo.pacman.adapter.picture.game.LogicScorePicture;
import italo.pacman.picture.AbstractComponentPicture;
import italo.pacman.picture.ContainerPicture;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;

public class ScorePicture extends AbstractComponentPicture implements LogicScorePicture {

	public final static double FONT_SIZE_FACTOR = 2.0;
	public final static double HEIGHT_FACTOR = 0.9;
	
	public final static Color LABEL_COLOR = Color.WHITE;
	public final static Color VALUE_COLOR = Color.GREEN;
	
	public final static Color DANGER_COLOR = Color.RED;
	
	public final static int DISTANCE_PIXELS_FACTOR = 20;
	
	private ScoreFigure scoreFigure;
	
	public ScorePicture(ContainerPicture container) {
		super(container);
	}

	public void init(ScoreFigure scoreFigure) {
		super.imagePicture.setComponent(scoreFigure);
		this.scoreFigure = scoreFigure;
	}
	
	public void paint(Graphics g) {
		int x = scoreFigure.getX();
		int y = scoreFigure.getY();
		int height = scoreFigure.getHeight();
		
		int fontSize  = (int) ( ( (double)height ) / FONT_SIZE_FACTOR );
		int heightNew = (int) ( ( (double)height ) * HEIGHT_FACTOR );
		int xNew = x;
		int yNew = y + heightNew;
		
		Font font = new Font("Bernard MT Condensed", Font.BOLD, fontSize);
		g.setFont(font);
		FontMetrics fontMetrics = g.getFontMetrics();	
				
		String lifeL  = " Vida(s)";
		String faseL  = "º Fase"; 
		String scoreL = " Pontos"; 
		String bollsL = " Bolinhas Restantes"; 
				
		String life  = String.valueOf( scoreFigure.getLifeQuantity()   );
		String fase  = String.valueOf( scoreFigure.getFaseNumber()     ); 
		String score = String.valueOf( scoreFigure.getScoreValue()     ); 
		String bolls = String.valueOf( scoreFigure.getRemainderBolls() ); 								
					
		int widthLife = fontMetrics.stringWidth(life);
		int widthFase = fontMetrics.stringWidth(fase);
		int widthScore = fontMetrics.stringWidth(score);
		int widthBolls = fontMetrics.stringWidth(bolls);

		int widthLifeL = fontMetrics.stringWidth(lifeL);
		int widthFaseL = fontMetrics.stringWidth(faseL);
		int widthScoreL = fontMetrics.stringWidth(scoreL);
				
		g.setColor(scoreFigure.getLifeQuantity() > 1 ? VALUE_COLOR : DANGER_COLOR);
		g.drawString(life   , xNew , yNew);
		xNew += widthLife;
		g.setColor(LABEL_COLOR);
		g.drawString(lifeL  , xNew , yNew);
		xNew += widthLifeL + DISTANCE_PIXELS_FACTOR;
		g.setColor(VALUE_COLOR);
		g.drawString(fase   , xNew , yNew);
		xNew += widthFase;
		g.setColor(LABEL_COLOR);
		g.drawString(faseL  , xNew , yNew);
		xNew += widthFaseL + DISTANCE_PIXELS_FACTOR;
		g.setColor(VALUE_COLOR);
		g.drawString(score  , xNew , yNew);
		xNew += widthScore;
		g.setColor(LABEL_COLOR);
		g.drawString(scoreL , xNew , yNew);
		xNew += widthScoreL + DISTANCE_PIXELS_FACTOR;
		g.setColor(VALUE_COLOR);
		g.drawString(bolls  , xNew , yNew);
		xNew += widthBolls;
		g.setColor(LABEL_COLOR);
		g.drawString(bollsL , xNew , yNew);
	}

}
