package italo.pacman.picture.game.tray;


import italo.pacman.adapter.figure.game.TrayFigure;
import italo.pacman.adapter.picture.game.LogicBigBollPicture;
import italo.pacman.adapter.picture.game.LogicBollPicture;
import italo.pacman.adapter.picture.game.LogicCellPicture;
import italo.pacman.adapter.picture.game.LogicHousePicture;
import italo.pacman.adapter.picture.game.LogicMessagePicture;
import italo.pacman.adapter.picture.game.LogicMonsterPicture;
import italo.pacman.adapter.picture.game.LogicPacmanPicture;
import italo.pacman.adapter.picture.game.LogicTrayPicture;
import italo.pacman.picture.AbstractContainerPicture;
import italo.pacman.picture.ComponentPicture;
import italo.pacman.picture.ContainerPicture;

import java.awt.Graphics;
import java.util.LinkedList;
import java.util.List;

public class TrayPicture extends AbstractContainerPicture implements LogicTrayPicture {

	private CellPicture cellPicture = new CellPicture(this);	
	private BollPicture bollPicture = new BollPicture(this);
	private BigBollPicture bigBollPicture = new BigBollPicture(this);
	private HousePicture housePicture = new HousePicture(this);
	private MessagePicture messagePicture = new MessagePicture(this);
	
	private PacmanPicture pacmanPicture = new PacmanPicture(this);
	private List<MonsterPicture> monstersPicture = new LinkedList<MonsterPicture>();
	
	private TrayFigure trayFigure;
	
	public TrayPicture(ContainerPicture container) {
		super(container);
	}

	public void init(TrayFigure trayFigure) {
		this.trayFigure = trayFigure;
		this.monstersPicture.clear();
		super.imagePicture.setComponent( trayFigure );
		super.removeAll();		
		super.add( cellPicture );
		super.add( bollPicture );
		super.add( bigBollPicture );
		super.add( housePicture );		
		for(int i = 0; i < trayFigure.getMonstersFigure().size(); i++) {
			MonsterPicture monsterPicture = new MonsterPicture(this);			
			this.monstersPicture.add( monsterPicture );			
			super.add( monsterPicture );
		}
		super.add( pacmanPicture );		
		super.add( messagePicture );
	}
	
	public void paint(Graphics g) {
		trayPaint(g);
		for(ComponentPicture component:components)
			component.paint(g);
	}
	
	private void trayPaint(Graphics g) {
		int x = trayFigure.getX();
		int y = trayFigure.getY();
		int width = trayFigure.getWidth();
		int height = trayFigure.getHeight();
		int widthWall = trayFigure.getWidthWall();
		
		trayFigure.setX( x - widthWall/2 );
		trayFigure.setY( y - widthWall/2 );
		trayFigure.setWidth(  width  + widthWall );
		trayFigure.setHeight( height + widthWall );
		
		super.imagePicture.paint(g);
		
		trayFigure.setX(x);
		trayFigure.setY(y);
		trayFigure.setWidth(width);
		trayFigure.setHeight(height);	
				
		//this._paint(g);
		
	}
	
/*
	private void _paint(Graphics g) {
		int x = trayFigure.getX();
		int y = trayFigure.getY();
		int widthCell = trayFigure.getCellFigure().getWidth();
		int heightCell =  trayFigure.getCellFigure().getHeight();
		g.setColor(Color.BLUE);		
		for(int i = 0; i < trayFigure.getHorizontalWalls().length; i++) {
			boolean[] vetor = trayFigure.getHorizontalWalls()[i];
			for(int j = 0; j < vetor.length; j++) {
				if(!vetor[j])					
					g.drawLine(x+(j*widthCell), y+(i*heightCell), x+(j*widthCell)+widthCell, y+(i*heightCell));
			}
		}
		for(int i = 0; i < trayFigure.getVerticalWalls().length; i++) {
			boolean[] vetor = trayFigure.getVerticalWalls()[i];
			for(int j = 0; j < vetor.length; j++) {
				if(!vetor[j])					
					g.drawLine(x+(j*widthCell), y+(i*heightCell), x+(j*widthCell), y+(i*heightCell)+heightCell);
			}
		}
	}
*/
	
	public LogicCellPicture getCellPicture() {
		return cellPicture;
	}

	public LogicBollPicture getBollPicture() {
		return bollPicture;
	}

	public LogicBigBollPicture getBigBollPicture() {
		return bigBollPicture;
	}

	public LogicHousePicture getHousePicture() {
		return housePicture;
	}

	public LogicMessagePicture getMessagePicture() {
		return messagePicture;
	}

	public LogicPacmanPicture getPacmanPicture() {
		return pacmanPicture;
	}

	public List<LogicMonsterPicture> getMonstersPicture() {
		List<LogicMonsterPicture> list = new LinkedList<LogicMonsterPicture>();
		list.addAll(monstersPicture);
		return list;
	}

	public TrayFigure getTrayFigure() {
		return trayFigure;
	}

}
